/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.actions;

/**
 * Add attribute action. 
 */

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.ui.actions.*;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.*;


import VOCLModel.ClassifierRole;
import VOCLModel.Attribute;
import VOCLModel.VariableDeclaration;
import vocleditor.VOCLEditorPlugin;
import vocleditor.commands.CreateAttributeCommand;
import vocleditor.dialog.AttributePropertiesDialog;
import vocleditor.edit.VOCLClassifierRoleEditPart;
import vocleditor.edit.AttributeEditPart;
import vocleditor.model.ModelCreationFactory;


/**
 * @author tugstugi, Stefan
 */
public class AddAttributeAction extends SelectionAction {
	static public String ID = "Add_Attribute_Action";
	private Shell shell;
	private	Attribute attribute;
	
	/**
	 * The default constructor with the workbench part.
	 * @param part the workbench part
	 */
	public AddAttributeAction(IWorkbenchPart part) {
		super(part);
		setDescription("Add Attribute Action");
		setText("Add Attribute");
	}
	
	/**
	 * Runs the action.
	 */
	public void run() {
		CreateAttributeCommand command = (CreateAttributeCommand) getCommand();
		if(command == null)
			return;
		Attribute attribute = (Attribute) (new ModelCreationFactory(Attribute.class)).getNewObject();
		command.setAttribute(attribute);
		AttributePropertiesDialog apd = new AttributePropertiesDialog( new Shell(), VOCLEditorPlugin.getConstraint(command.getParent()), command.getParent(), attribute);
		apd.open();
		if( apd.getReturnCode() == Dialog.CANCEL ) 
			return;
		VariableDeclaration variable = null;
		if(apd.getVariableName() != null && !"".equals(apd.getVariableName())){
			variable = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
			variable.setName(apd.getVariableName());
			variable.setType(apd.getType());
		}
		command.setCommandAttributes(apd.getName(),apd.getType(),apd.getValue(), variable);
		execute(command);
	}
	
	/**
	 * Gets the add attribute command.
	 * @return the add attribute command
	 */
	public Command getCommand()	{
		Object selection = ((StructuredSelection) getSelection()).getFirstElement();		
		AbstractGraphicalEditPart editpart = null;
		if (selection instanceof VOCLClassifierRoleEditPart){
			editpart = (VOCLClassifierRoleEditPart) selection;
			CreateAttributeCommand addAttributeCommand = new CreateAttributeCommand();
			addAttributeCommand.setParent((ClassifierRole) editpart.getModel());
			return addAttributeCommand;
		}else if (selection instanceof AttributeEditPart) {
			editpart = (VOCLClassifierRoleEditPart) ((AttributeEditPart) selection).getParent();
			CreateAttributeCommand addAttributeCommand = new CreateAttributeCommand();
			addAttributeCommand.setParent((ClassifierRole) editpart.getModel());
			return addAttributeCommand;			
		}
		return null;		
	}
	
	/**
	 * Returns true if the action is enabled. 
	 */
	protected boolean calculateEnabled(){	
		return canPerformAction();
	}
	
	/**
	 * Returns true if the action could be executed (a ClassifierRole is selected), false otherwise.
	 */
	private boolean canPerformAction(){
		if (getSelectedObjects().isEmpty())
			return false;
		List parts = getSelectedObjects();
		for (int i = 0; i < parts.size(); i++) {
			Object o = parts.get(i);
			if (!(o instanceof EditPart))
				return false;
			EditPart part = (EditPart) o;
			if (part.getModel() instanceof ClassifierRole)
				return true;
			else if (part.getModel() instanceof Attribute)
				return true;
			else
				return false;
		}
		return true;
	}

	/**
	 * Inits the action.
	 */
	protected void init() {
		super.init();
		setId(ID);
	}
}
