/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.actions;

/**
 * Add condition action. 
 */

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.ui.actions.*;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.*;


import VOCLModel.*;
import vocleditor.VOCLEditorPlugin;
import vocleditor.commands.CreateConditionCommand;
import vocleditor.dialog.ConditionPropertiesDialog;
import vocleditor.edit.NavExpEditPart;
import vocleditor.model.ModelCreationFactory;


/**
 * @author tugstugi, manaljav
 * Add condition action.
 */
public class AddConditionAction extends SelectionAction {
	static public String ID = "Add_Condition_Action";
	private Shell shell;
	private	Condition condition;
	
	/**
	 * The default constructor with the workbench part.
	 * @param part the workbench part
	 */
	public AddConditionAction(IWorkbenchPart part) {
		super(part);
		setDescription("Add Condition Action");
		setText("Add Condition");
	}
	
	/**
	 * Runs the action.
	 */
	public void run() {
		CreateConditionCommand command = (CreateConditionCommand) getCommand();
		if(command == null)
			return;
		Condition condition = (Condition) (new ModelCreationFactory(Condition.class)).getNewObject();
		command.setCondition(condition);
		ConditionPropertiesDialog cpd = new ConditionPropertiesDialog( new Shell(), condition, VOCLEditorPlugin.getConstraint(command.getParent()), command.getParent());
		cpd.open();
		if( cpd.getReturnCode() == Dialog.CANCEL ) 
			return;
		command.setCommandAttributes(cpd.getOp(),cpd.getValue(),cpd.getLeft(),cpd.getRight(),cpd.getLeftPre(),cpd.getRightPre());
		
		execute(command);
	}
	
	/**
	 * Gets the add condition command.
	 * @return the add condition command
	 */
	public Command getCommand()	{
		Object selection = ((StructuredSelection) getSelection()).getFirstElement();		
		AbstractGraphicalEditPart editpart = null;
		if (selection instanceof NavExpEditPart && ((NavExp) ((NavExpEditPart) selection).getModel()).getCondition() == null){
			editpart = (NavExpEditPart) selection;
			CreateConditionCommand addConditionCommand = new CreateConditionCommand();
			addConditionCommand.setParent((NavExp) editpart.getModel());
			return addConditionCommand;
		}
		return null;		
	}
	
	/**
	 * Returns true if the action is enabled. 
	 */
	protected boolean calculateEnabled(){	
		return canPerformAction();
	}
	
	/**
	 * Returns true if the action could be executed , false otherwise.
	 */
	private boolean canPerformAction(){
		if (getSelectedObjects().isEmpty())
			return false;
		List parts = getSelectedObjects();
		for (int i = 0; i < parts.size(); i++) {
			Object o = parts.get(i);
			if (!(o instanceof EditPart))
				return false;
			EditPart part = (EditPart) o;
			if (part.getModel() instanceof NavExp && ((NavExp)part.getModel()).getCondition()==null)
				return true;
			else
				return false;
		}
		return true;
	}

	/**
	 * Inits the action.
	 */
	protected void init() {
		super.init();
		setId(ID);
	}
}
