/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.actions;

/**
 * Add attribute action. 
 */
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.ui.actions.*;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.*;


import VOCLModel.ClassifierRole;
import VOCLModel.Method;
import VOCLModel.SetClassifierRole;
import VOCLModel.VariableDeclaration;
import vocleditor.VOCLEditorPlugin;
import vocleditor.commands.CreateMethodCommand;
import vocleditor.dialog.MethodPropertiesDialog;
import vocleditor.edit.VOCLClassifierRoleEditPart;
import vocleditor.model.ModelCreationFactory;


/**
 * @author tugstugi
 */
public class AddMethodAction extends SelectionAction {
	static public String ID = "Add_Method_Action";
	private Shell shell;
	private	Method method;
	
	/**
	 * The default constructor with the workbench part.
	 * @param part the workbench part
	 */
	public AddMethodAction(IWorkbenchPart part) {
		super(part);
		setDescription("Add Method Action");
		setText("Add Method");
	}
	
	/**
	 * Runs the action.
	 */
	public void run() {
		CreateMethodCommand command = (CreateMethodCommand) getCommand();
		if(command == null)
			return;
		Method method = (Method) (new ModelCreationFactory(Method.class)).getNewObject();
		MethodPropertiesDialog mpd = new MethodPropertiesDialog(new Shell(), VOCLEditorPlugin.getConstraint(command.getParent()), command.getParent(), method);
		mpd.open();
		if( mpd.getReturnCode() == Dialog.CANCEL ) 
			return;
		command.setMethod(method);
		command.setName(mpd.getName());
		command.setReturnType(mpd.getReturnType());
		VariableDeclaration var = null;
		String varName = mpd.getReturnVariableName();
		if ( varName != null ) {
			var = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
			var.setName( varName );
			var.setType( mpd.getReturnType() );
			//var.setReturningMethod( method );#
			VOCLEditorPlugin.getConstraint( command.getParent() ).getAllVariables().add( var );
		}
		command.setReturnVariable( var );
		command.setParameters(mpd.getParameters());
		execute(command);
	}
	
	/**
	 * Gets the add method command.
	 * @return the add method command
	 */
	public Command getCommand()	{
		Object selection = ((StructuredSelection) getSelection()).getFirstElement();		
		AbstractGraphicalEditPart editpart = null;
		if (selection instanceof VOCLClassifierRoleEditPart){
			editpart = (VOCLClassifierRoleEditPart) selection;
			CreateMethodCommand addMethodCommand = new CreateMethodCommand();
			addMethodCommand.setParent((ClassifierRole) editpart.getModel());
			return addMethodCommand;
		}
		return null;		
	}
	
	/**
	 * Returns true if the action is enabled. 
	 */
	protected boolean calculateEnabled(){	
			return canPerformAction();
	}
	
	/**
	 * Returns true if the action could be executed (a ClassifierRole is selected), false otherwise.
	 */
	private boolean canPerformAction(){
		if (getSelectedObjects().isEmpty())
			return false;
		List parts = getSelectedObjects();
		for (int i = 0; i < parts.size(); i++) {
			Object o = parts.get(i);
			if (!(o instanceof EditPart))
				return false;
			EditPart part = (EditPart) o;
			if (part.getModel() instanceof ClassifierRole && ! (part.getModel() instanceof SetClassifierRole) && ((ClassifierRole) part.getModel()).getMethod() == null)
				return true;
			else
				return false;
		}
		return true;
	}

	/**
	 * Inits the action.
	 */
	protected void init() {
		super.init();
		setId(ID);
	}
}
