/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.actions;

/**
 * Convert VOCL2OCL action. 
 */
import java.io.IOException;

import javax.xml.transform.TransformerException;

import org.eclipse.core.resources.IFile;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.*;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.*;
import org.eclipse.ui.part.FileEditorInput;

import vocleditor.converter.Converter;
import vocleditor.dialog.OCLStringDialog;


/**
 * @author tugstugi
 */
public class ConvertAction extends SelectionAction {
	static public String ID = "Convert_Action";
	
	/**
	 * The default constructor with the workbench part.
	 * @param part the workbench part
	 */
	public ConvertAction(IWorkbenchPart part) {
		super(part);
		setDescription("Convert Action");
		setText("Convert to OCL");
	}
	
	/**
	 * Runs the action.
	 */
	public void run() {
		IWorkbench workbench = PlatformUI.getWorkbench();
		IWorkbenchWindow workbenchWindow = workbench.getActiveWorkbenchWindow();
		IWorkbenchPage workbenchPage = workbenchWindow.getActivePage();
		IEditorPart editorPart = workbenchPage.getActiveEditor();
		IFile file = ((FileEditorInput) editorPart.getEditorInput()).getFile();
		try {
			Converter converter = new Converter(file.getLocation().toFile());
			converter.convert();
			OCLStringDialog dialog = new OCLStringDialog(new Shell(), converter.getOCLString());
			dialog.open();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Gets the add attribute command.
	 * @return the add attribute command
	 */
	public Command getCommand()	{		
		return null;		
	}
	
	/**
	 * Returns true if the action is enabled. 
	 */
	protected boolean calculateEnabled(){	
		return canPerformAction();
	}
	
	/**
	 * Returns true.
	 */
	private boolean canPerformAction(){
		return true;
	}

	/**
	 * Inits the action.
	 */
	protected void init() {
		super.init();
		setId(ID);
	}
}
