/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.actions;

/**
 * Edit action for the properties dialog,
 * where the user can change the ClassifierRole or AssociationRole properties.
  */
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractEditPart;
import org.eclipse.gef.ui.actions.*;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.*;


import VOCLModel.*;
import vocleditor.VOCLEditorPlugin;
import vocleditor.commands.*;
import vocleditor.dialog.*;
import vocleditor.edit.*;



/**
 * @author Stefan
 *
 */
public class EditAction extends SelectionAction {
	static public String ID = "Properties_Dialog_Action";
	private Shell shell;
	private AbstractEditPart editpart;

	/**
	 * The default constructor with the workbench part.
	 * @param part the workbench part
	 */
	public EditAction(IWorkbenchPart part) {
		super(part);
		setDescription("Properties Dialog Action");
		setText("Properties Dialog");
	}
	
	/**
	 * Runs the action.
	 */
	public void run() {
		Command command = null;
		Object selection = ((StructuredSelection) getSelection()).getFirstElement();
		if (selection instanceof VOCLClassifierRoleEditPart){
			editpart = (VOCLClassifierRoleEditPart)selection;			
			ClassifierRole cr = (ClassifierRole) editpart.getModel();
			if(cr instanceof SetClassifierRole){
				command = new EditSetClassifierRoleCommand();
				((EditSetClassifierRoleCommand)command).setClassifierRole((SetClassifierRole) cr);			
				SetClassifierRolePropertiesDialog cpd = new SetClassifierRolePropertiesDialog( new Shell(), VOCLEditorPlugin.getConstraint(cr), (SetClassifierRole) cr);
				cpd.open();
				if( cpd.getReturnCode() == Dialog.CANCEL ) 
					return;
				((EditSetClassifierRoleCommand)command).setCommandAttributes(cpd.getClassifierName(), cpd.getClassifierType(), cpd.getOpKind(), cpd.getSizeVariable(), cpd.getIteratorVariable(), cpd.getSumVariable(), cpd.getChangedVariables(), cpd.getAttributes());
			}else{
				command = new EditClassifierRoleCommand();
				((EditClassifierRoleCommand)command).setClassifierRole(cr);			
				ClassifierPropertiesDialog cpd = new ClassifierPropertiesDialog( new Shell(), VOCLEditorPlugin.getConstraint(cr), cr);
				cpd.open();
				if( cpd.getReturnCode() == Dialog.CANCEL ) 
					return;
				((EditClassifierRoleCommand)command).setCommandAttributes(cpd.getClassifierName(),cpd.getClassifierType(),cpd.getChangedVariables(),cpd.getAttributes());
			}
		}else if (selection instanceof VOCLAssociationRoleEditPart){
			editpart = (VOCLAssociationRoleEditPart)selection;
			command = new EditAssociationRoleCommand();
			AssociationRole ar = (AssociationRole) editpart.getModel();
			((EditAssociationRoleCommand)command).setAssociationRole(ar);			
			AssociationPropertiesDialog d = new AssociationPropertiesDialog(new Shell(), ar );
			d.open();
			if( d.getReturnCode() == Dialog.CANCEL )
				return;
			((EditAssociationRoleCommand)command).setCommandAttributes(d.getSource(),d.getTarget(),d.getName(), ar.getRoutingType());
		}else if (selection instanceof AttributeEditPart){
			editpart = (AttributeEditPart) selection;
			VOCLClassifierRoleEditPart editpartParent = (VOCLClassifierRoleEditPart) editpart.getParent();
			command = new EditAttributeCommand();
			Attribute att = (Attribute)editpart.getModel();
			((EditAttributeCommand)command).setAttribute(att);
			((EditAttributeCommand)command).setParent((ClassifierRole) editpartParent.getModel());
			AttributePropertiesDialog apd = new AttributePropertiesDialog( new Shell(), VOCLEditorPlugin.getConstraint(editpartParent.getClassifierRole()), editpartParent.getClassifierRole(), att);
			apd.open();
			if( apd.getReturnCode() == Dialog.CANCEL)
				return;
			VariableDeclaration variable = null;
			((EditAttributeCommand)command).setCommandAttributes(apd.getName(), apd.getType(), apd.getValue(), apd.getVariableName());
		}if (selection instanceof MethodEditPart){
			editpart = (MethodEditPart) selection;
			command = new EditMethodCommand();
			Method method = (Method) editpart.getModel();
			((EditMethodCommand) command).setMethod(method);			
			MethodPropertiesDialog mpd = new MethodPropertiesDialog( new Shell(), VOCLEditorPlugin.getConstraint(method.getClassifierRole()) , method.getClassifierRole(), method);
			mpd.open();
			if( mpd.getReturnCode() == Dialog.CANCEL ) 
				return;
			((EditMethodCommand) command).setCommandAttributes(mpd.getName(), mpd.getReturnType(), mpd.getReturnVariableName(), mpd.getChangedParameters(), mpd.getParameters());
		}else if (selection instanceof ConditionEditPart){
			editpart = (ConditionEditPart) selection;
			command = new EditConditionCommand();
			Condition condition = (Condition)editpart.getModel();
			((EditConditionCommand)command).setCondition(condition);			
			NavExp navExp = (NavExp) ((NavExpEditPart) editpart.getParent()).getModel();
			Constraint constraint = VOCLEditorPlugin.getConstraint(navExp);
			ConditionPropertiesDialog cpd = new ConditionPropertiesDialog( new Shell(), condition , constraint, navExp);
			cpd.open();
			if( cpd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditConditionCommand)command).setCommandAttributes(cpd.getOp(),cpd.getValue(),cpd.getLeft(),cpd.getRight(),cpd.getLeftPre(),cpd.getRightPre());
		} else if (selection instanceof ContextEditPart) {
			editpart = (ContextEditPart) selection;
			Context context = (Context)editpart.getModel();
			command = new EditContextCommand( context );
			ContextPropertiesDialog cpd = new ContextPropertiesDialog( new Shell(), context );
			cpd.open();
			if( cpd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditContextCommand)command).setCommandAttributes(cpd.getContextName(), cpd.getContextType(),cpd.getIsClassifier(),cpd.getContextClassifier(), cpd.getContextMethod() );
		} else if (selection instanceof BoolExpEditPart ) {
			editpart = (BoolExpEditPart) selection;
			BoolExp boolexp = (BoolExp)editpart.getModel();
			command = new EditBoolExpCommand( boolexp );
			BoolExpPropertiesDialog bpd = new BoolExpPropertiesDialog( new Shell(), boolexp );
			bpd.open();
			if( bpd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditBoolExpCommand)command).setCommandAttributes( bpd.getOperation(), bpd.getIsHorizontal(), bpd.getSplitPercentage(), bpd.isNegated() );
		} else if (selection instanceof IfExpEditPart ) {
			editpart = (IfExpEditPart) selection;
			IfExp ifexp = (IfExp)editpart.getModel();
			command = new EditIfExpCommand( ifexp );
			IfExpPropertiesDialog ipd = new IfExpPropertiesDialog( new Shell(), ifexp );
			ipd.open();
			if( ipd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditIfExpCommand)command).setCommandAttributes(  ipd.getIsHorizontal(), ipd.getSplitPercentage1(), ipd.getSplitPercentage2(), ipd.isNegated() );
		} else if (selection instanceof LetExpEditPart) {
			editpart = (LetExpEditPart) selection;
			LetExp letexp = (LetExp)editpart.getModel();
			command = new EditLetExpCommand( letexp );
			LetExpPropertiesDialog bpd = new LetExpPropertiesDialog( new Shell(), letexp );
			bpd.open();
			if( bpd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditLetExpCommand)command).setCommandAttributes( bpd.getIsHorizontal(), bpd.getSplitPercentage(), bpd.getVariableDeclaration(), bpd.isNegated() );
		} else if (selection instanceof NavExpEditPart ) {
			editpart = (AbstractEditPart)((NavExpEditPart) selection).getParent();
			if  (editpart instanceof VOCLConstraintEditPart) return;
			else if (editpart instanceof LetExpEditPart ) {
			LetExp letexp = (LetExp)editpart.getModel();
			command = new EditLetExpCommand( letexp );
			LetExpPropertiesDialog bpd = new LetExpPropertiesDialog( new Shell(), letexp );
			bpd.open();
			if( bpd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditLetExpCommand)command).setCommandAttributes( bpd.getIsHorizontal(), bpd.getSplitPercentage(), bpd.getVariableDeclaration(), bpd.isNegated() );
			}
			else if (editpart instanceof IfExpEditPart ) {
				IfExp ifexp = (IfExp)editpart.getModel();
				command = new EditIfExpCommand( ifexp );
				IfExpPropertiesDialog ipd = new IfExpPropertiesDialog( new Shell(), ifexp );
				ipd.open();
				if( ipd.getReturnCode() == Dialog.CANCEL)
					return;
				((EditIfExpCommand)command).setCommandAttributes(  ipd.getIsHorizontal(), ipd.getSplitPercentage1(), ipd.getSplitPercentage2(), ipd.isNegated() );
			}
			else if (editpart instanceof BoolExpEditPart ) {
				BoolExp boolexp = (BoolExp)editpart.getModel();
				command = new EditBoolExpCommand( boolexp );
				BoolExpPropertiesDialog bpd = new BoolExpPropertiesDialog( new Shell(), boolexp );
				bpd.open();
				if( bpd.getReturnCode() == Dialog.CANCEL)
					return;
				((EditBoolExpCommand)command).setCommandAttributes( bpd.getOperation(), bpd.getIsHorizontal(), bpd.getSplitPercentage(), bpd.isNegated() );
			}
			
		}

		if(command == null)
			return;
		execute(command);
	}	

	/**
	 * Returns true if the action is enabled. 
	 */
	protected boolean calculateEnabled() {	
			return canPerformAction();
	}
	
	/**
	 * Returns true if the action could be executed 
	 *  (a ClassifierRole or AssociationRole ist selected), false otherwise.
	 */
	private boolean canPerformAction() {
		if (getSelectedObjects().isEmpty())
			return false;
		List parts = getSelectedObjects();
		for (int i = 0; i < parts.size(); i++) {
			Object o = parts.get(i);
			if (!(o instanceof EditPart))
				return false;
			EditPart part = (EditPart) o;
			if ((part.getModel() instanceof ClassifierRole) ||
				(part.getModel() instanceof AssociationRole) ||
				(part.getModel() instanceof Attribute) ||
				(part.getModel() instanceof Method) ||
			    (part.getModel() instanceof BoolExp ) ||
			    (part.getModel() instanceof IfExp ) ||
			    (part.getModel() instanceof LetExp ) ||
			    (part.getModel() instanceof Condition ) ||
			    (part.getModel() instanceof Label ) ||
			    (part.getModel() instanceof NavExp && !(part.getParent().getModel() instanceof Constraint)  ) ||
			    (part.getModel() instanceof Context )) 
					return true;
				return false;
		}
		return true;
	}

	/**
	 * Inits the action.
	 */
	protected void init() {
		super.init();
		setId(ID);
	}
}
