/*
 * Created on 28.06.2005
 *
 */
package vocleditor.actions;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;

import VOCLModel.Label;
import vocleditor.edit.LabelGraphicalEditPart;
import vocleditor.commands.ResetLabelCommand;

/**
 * SelectionAction for resetting AssociationRole's label position
 * @author jan
 */
public class ResetLabelPositionAction extends SelectionAction {
	static public String ID = "Reset_Label_Position";
	private Label label;

	/**
	 * The default constructor with the workbench part.
	 * @param part the workbench part
	 */
	public ResetLabelPositionAction(IWorkbenchPart part) {
		super(part);
		setDescription("Reset Label Position Action");
		setText("Reset Position");
	}

	/**
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		return canPerformAction();
	}

	/**
	 * Runs the action.
	 */
	public void run() {
		Object selection = ((StructuredSelection) getSelection()).getFirstElement();
		if(selection instanceof LabelGraphicalEditPart) {
			ResetLabelCommand command = new ResetLabelCommand((Label)((LabelGraphicalEditPart)selection).getModel());
			execute(command);
		}
	}
	
	/**
	 * Returns true if the action could be executed (a Label is selected) and this label has customPosition set to false, false otherwise.
	 */
	private boolean canPerformAction(){
		if (getSelectedObjects().isEmpty())
			return false;
		List parts = getSelectedObjects();
		for (int i = 0; i < parts.size(); i++) {
			Object o = parts.get(i);
			if (!(o instanceof EditPart))
				return false;
			EditPart part = (EditPart) o;
			if (part.getModel() instanceof Label && ((Label)part.getModel()).isCustomPosition())
				return true;
			else
				return false;
		}
		return true;
	}

	/**
	 * Inits the action.
	 */
	protected void init() {
		super.init();
		setId(ID);
	}

}
