/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import org.eclipse.gef.commands.Command;
import org.eclipse.swt.widgets.Shell;

import vocleditor.model.ModelCreationFactory;

import VOCLModel.ClassifierRole;
import VOCLModel.NavExp;
import VOCLModel.AssociationRole;
import VOCLModel.Label;

/**
 * Connects two ClassifierRoles via an AssociationRole.
 */
public class CreateAssociationRoleCommand extends Command {
	protected ClassifierRole source;
	protected ClassifierRole target;
	protected AssociationRole associationrole;
	private Shell shell;
	private String name;
	
	/**
	 * Constructor for the command class.
	 */
	public CreateAssociationRoleCommand() {
		setLabel("create association role");
	}

	/**
	 * True if the command could be executed, false otherwise.
	 */
	public boolean canExecute() {
		if (source == null || target == null || associationrole == null || source == target)
			return false;
		else return true; 
	}

	/**
	 * Entry point for command execution. Will create a {@link Label} and set it.
	 */
	public void execute() {
		Label label = (Label)(new ModelCreationFactory(Label.class)).getNewObject();
		associationrole.setLabelDecoration(label);
		NavExp navexp = target.getNavExp();
		associationrole.setName(name);
		associationrole.setNavExp( navexp );
		navexp.getAssociationRoles().add(associationrole);
		source.getTarget().add(associationrole);
		target.getSource().add(associationrole);
		associationrole.setAssociationSource(source);
		associationrole.setAssociationTarget(target);
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo() {
		NavExp navexp = target.getNavExp();
		navexp.getAssociationRoles().add(associationrole);
		source.getTarget().add(associationrole);
		target.getSource().add(associationrole);
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		target.getNavExp().getAssociationRoles().remove(associationrole);
		source.getTarget().remove(associationrole);
		target.getSource().remove(associationrole);	
	}
		
	/**
	 * Returns the source classifier role.
	 * @return the source classifier role
	 */
	public ClassifierRole getSource() {
		return source;
	}
	
	/**
	 * Returns the association role.
	 * @return the association role
	 */
	public AssociationRole getAssociationRole(){
		return associationrole;
	}
	
	/**
	 * Returns the target classifier role.
	 * @return the target classifier role
	 */
	public ClassifierRole getTarget() {
		return target;
	}

	/**
	 * Sets the source classifier role.
	 * @param newSource the source classifier role to set
	 */
	public void setSource(ClassifierRole newSource) {
		source = newSource;
	}

	/**
	 * Sets the target classifier role.
	 * @param newTarget the target classifier role to set
	 */
	public void setTarget(ClassifierRole newTarget) {
		target = newTarget;
	}
	
	/**
	 * Sest the association role.
	 * @param associationRole the association role to set
	 */
	public void setAssociationRole(AssociationRole associationRole){
		this.associationrole = associationRole;
	}
	
	/**
	 * Sets the name of the association role.
	 * @param name the name of the association role
	 */
	public void setName (String name) {
		this.name=name;
	}
	
	/**
	 * Sets the command attributes.
	 * @param source the source classifier role
	 * @param target the target classifier role
	 * @param name the name of the association role
	 */
	public void setCommandAttributes (ClassifierRole source, ClassifierRole target, String name) {
		setSource(source);
		setTarget(target);
		setName(name);		
	}	
}
