/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import org.eclipse.swt.widgets.Shell;

import vocleditor.VOCLEditorPlugin;

import VOCLModel.Attribute;
import VOCLModel.ClassifierRole;
import VOCLModel.VariableDeclaration;


/**
 * @author tugstugi
 * Command for creating an attribute.
 */
public class CreateAttributeCommand	extends org.eclipse.gef.commands.Command {
	private	Attribute attribute;
	private ClassifierRole parent;
	private Shell shell;
	private String name, type, value;
	private VariableDeclaration variable;
	
	/**
	 * The default constructor.
	 */	
	public CreateAttributeCommand() {
		setLabel("create attribute");
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() {   
		parent.getAttributes().add(attribute);
		attribute.setName(name);
		attribute.setType(type);
		attribute.setValue(value);
		attribute.setVariable(variable);
		if(variable != null){
			VOCLEditorPlugin.getConstraint( parent).getAllVariables().add(variable);
		}
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		parent.getAttributes().remove(attribute);
		if(variable != null)
			VOCLEditorPlugin.getConstraint( parent).getAllVariables().remove(variable);
	}
	
	/**
	 * Returns the parent classifier role.
	 * @return the parent classifier role
	 */
	public ClassifierRole getParent() {
		return parent;
	}	
	
	/**
	 * Sets the parent classifier role.
	 * @param newParent the parent to set
	 */
	public void setParent(ClassifierRole newParent) {
		parent = newParent;
	}
	
	/**
	 * Sets the attribute.
	 * @param attribute the attribute to set
	 */
	public void setAttribute(Attribute attribute ) {
		this.attribute = attribute;
	}	
		
	/**
	 * Sets the attribute name.
	 * @param name the attribute name to set
	 */
	public void setName(String name) {
		this.name=name;
	}
	
	/**
	 * Sets the attribute type.
	 * @param type the attribute type to set
	 */
	public void setType(String type) {
		this.type=type;
	}
	
	/**
	 * Sets the attribute value.
	 * @param value the attribute value to set
	 */
	public void setValue(String value) {
		this.value=value;
	}
	
	/**
	 * Sets the attribute variable.
	 * @param variable the attribute variable to set
	 */
	public void setVariable(VariableDeclaration variable){
		this.variable = variable;
	}
	
	/**
	 * Sets the command attributes.
	 * @param name the attribute name
	 * @param type the attribute type
	 * @param value the attribute value
	 * @param variable the attribute variable
	 */
	public void setCommandAttributes(String name, String type, String value, VariableDeclaration variable) {
		setName(name);
		setType(type);
		setValue(value);
		setVariable(variable);
	}
}
