/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;


import org.eclipse.draw2d.geometry.Point;
import org.eclipse.swt.widgets.Shell;

import VOCLModel.BoolExp;
import VOCLModel.BoolOperation;
import VOCLModel.NavExp;

/**
 * Command for creating a boolean expression role.
 * @author bjoern
 */
public class CreateBoolExpCommand extends org.eclipse.gef.commands.Command {
	private	BoolExp boolexp;
	private Point loc;
	private NavExp parent;
	private Shell shell;
	private BoolOperation operation;
	private boolean isHorizontal;
	private boolean isNegated;	
	private int splitPercentage;
	
	/**
	 * The default constructor.
	 */	
	public CreateBoolExpCommand( BoolExp boolexp, NavExp parent) 
	{
		setLabel("create bool expression");
		this.boolexp = boolexp;
		this.parent = parent;
	}
	
	/**
	 * Sets the bool expression.
	 * @return the bool expression
	 */
	public BoolExp getBoolExp()
	{
		return boolexp;
	}
	
	/**
	 * Sets the command attributes.
	 * @param operation the operation
	 * @param isHorizontal the horizontal flag
	 * @param splitPercentage the split percentage
	 * @param isNegated the flag shows the negation
	 */
	public void setCommandAttributes( BoolOperation operation, boolean isHorizontal, int splitPercentage, boolean isNegated )
	{
		this.operation = operation;
		this.splitPercentage = splitPercentage;
		this.isHorizontal = isHorizontal;
		this.isNegated = isNegated;
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() 
	{   
		parent.getExpressions().add( boolexp );
		if (loc != null) {
			boolexp.setX(loc.x);
			boolexp.setY(loc.y);
		}
		boolexp.setSplitHorizontal( isHorizontal );
		boolexp.setSplitPercent( splitPercentage );
		boolexp.setKind( operation );
		boolexp.setNegated( isNegated );
	}
		
	/**
	 * Redo command execution.
	 */
	public void redo() {
		parent.getExpressions().add( boolexp );
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		parent.getExpressions().remove( boolexp );
	}
	
	/**
	 * Sets the location.
	 * @param loc the location
	 */
	public void setLocation (Point loc) {
		this.loc = loc;
	}	
}
