/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import org.eclipse.gef.commands.Command;

import VOCLModel.*;

/**
 * @author manaljav
 * Command for creating a condition.
 */
public class CreateConditionCommand	extends Command {
	private	Condition condition;
	private NavExp parent;
	private String op, value;
	private VariableDeclaration left,right;
	private boolean leftPre, rightPre;
	
	/**
	 * The default constructor.
	 */	
	public CreateConditionCommand() {
		setLabel("create condition");
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() {   
		parent.setCondition(condition);
		condition.setOp(op);
		condition.setValue(value);
		condition.setRight(right);
		condition.setLeft(left);
		condition.setLeftPre(leftPre);
		condition.setRightPre(rightPre);
	}	
	
	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		parent.setCondition(null);
	}
	
	/**
	 * Sets the condition.
	 * @param condition the condition to set
	 */
	public void setCondition(Condition condition ) {
		this.condition = condition;
	}
	
	/**
	 * Returns the parent constraint.
	 * @return the parent constraint
	 */
	public NavExp getParent() {
		return parent;
	}
	
	/**
	 * Sets the parent constraint.
	 * @param newParent the parent to set
	 */
	public void setParent(NavExp newParent) {
		parent = newParent;
	}	
	
	/**
	 * Sets the operation. 
	 * @param op the op to set
	 */
	public void setOp(String op) {
		this.op = op;
	}
	
	/**
	 * Sets the value.
	 * @param value the value to set
	 */
	public void setValue(String value) {
		this.value = value;
	}
	
	/**
	 * Sets the right variable.
	 * @param right the right variable to set
	 */
	public void setRight(VariableDeclaration right) {
		this.right = right;
	}
	
	/**
	 * Sets the left variable.
	 * @param left the left variable to set
	 */
	public void setLeft(VariableDeclaration left) {
		this.left = left;
	}
	
	/**
	 * Sets the left pre.
	 * @param leftPre the left pre
	 */
	public void setLeftPre(boolean leftPre){
		this.leftPre = leftPre;		
	}
	
	/**
	 * Sets the rigth pre.
	 * @param rightPre the right pre
	 */
	public void setRightPre(boolean rightPre){
		this.rightPre = rightPre;
	}
	
	/**
	 * Sets the attributes.
	 * @param op the operation
	 * @param value the value
	 * @param left the left variable
	 * @param right the right variable
	 */
	public void setCommandAttributes(String op, String value, VariableDeclaration left, VariableDeclaration right, boolean leftPre, boolean rightPre) {
		setOp(op);
		setValue(value);
		setRight(right);
		setLeft(left);
		setLeftPre(leftPre);
		setRightPre(rightPre);
	}
}
