/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;


import org.eclipse.draw2d.geometry.Point;
import org.eclipse.swt.widgets.Shell;

import VOCLModel.LetExp;
import VOCLModel.NavExp;

/**
 * Command for creating a boolean expression role.
 * @author bjoern
 */
public class CreateLetExpCommand extends org.eclipse.gef.commands.Command {
	private	LetExp letexp;
	private Point loc;
	private NavExp parent;
	private Shell shell;
	private boolean isHorizontal;
	private boolean isNegated;
	private int splitPercentage;
	
	/**
	 * The default constructor.
	 */	
	public CreateLetExpCommand( LetExp letexp, NavExp parent) 
	{
		setLabel("create bool expression");
		this.letexp = letexp;
		this.parent = parent;
	}
	
	/**
	 * Returns the let expression.
	 * @return the let expression
	 */
	public LetExp getLetExp()
	{
		return letexp;
	}
	
	/**
	 * Sets the command attributes.
	 * @param isHorizontal the horizontal flag
	 * @param splitPercentage the split percentage
	 * @param isNegated the flag shows the negation
	 */
	public void setCommandAttributes( boolean isHorizontal, int splitPercentage, boolean isNegated )
	{
		this.splitPercentage = splitPercentage;
		this.isHorizontal = isHorizontal;
		this.isNegated = isNegated;
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() 
	{   
		parent.getExpressions().add( letexp );
		if (loc != null) {
			letexp.setX(loc.x);
			letexp.setY(loc.y);
		}
		letexp.setSplitHorizontal( isHorizontal );
		letexp.setSplitPercent( splitPercentage );
		letexp.setNegated( isNegated );
	}
		
	/**
	 * Redo command execution.
	 */
	public void redo() {
		parent.getExpressions().add( letexp );
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		parent.getExpressions().remove( letexp );
	}
		
	/**
	 * Sets the location.
	 * @param loc the location to set
	 */
	public void setLocation (Point loc) {
		this.loc = loc;
	}	
}
