/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;
import org.eclipse.swt.widgets.Shell;

import vocleditor.VOCLEditorPlugin;
import vocleditor.model.ModelCreationFactory;

import VOCLModel.Attribute;
import VOCLModel.SetClassifierRole;
import VOCLModel.NavExp;
import VOCLModel.VariableDeclaration;



/**
 * @author tugstugi
 * Command for creating a set classifier role.
 */
public class CreateSetClassifierRoleCommand extends Command {
	private	SetClassifierRole setClassifierRole;
	private Point loc;
	private NavExp parent;
	private Shell shell;
	private String name= "";
	private String type = "";
	private String opKind = "";
	private String size = "";
	private String iterator = "";
	private String sum = "";
	private List  attributes = new ArrayList();
	
	/**
	 * The default constructor.
	 */	
	public CreateSetClassifierRoleCommand() {
		setLabel("create set classifier role");
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() {   	
		parent.getClassifierRoles().add( setClassifierRole );		
		setClassifierRole.setName(name);
		setClassifierRole.setType(type);
		setClassifierRole.setOpKind(opKind);
		setClassifierRole.getAttributes().clear();
		for(int i=0; i<attributes.size(); i++){
			setClassifierRole.getAttributes().add(attributes.get(i));
			if(((Attribute) attributes.get(i)).getVariable() != null)
				VOCLEditorPlugin.getConstraint( setClassifierRole ).getAllVariables().add(((Attribute)attributes.get(i)).getVariable());
		}
		if("size".equals(opKind)){
			VariableDeclaration n = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
			n.setName(size);
			n.setType("int");
			setClassifierRole.setSize(n);
			VOCLEditorPlugin.getConstraint( setClassifierRole ).getAllVariables().add(n);
		}else if("sum".equals(opKind)){
			VariableDeclaration iteratorVariable = null;
			for(int i=0; i<setClassifierRole.getAttributes().size(); i++){
				if(((Attribute) setClassifierRole.getAttributes().get(i)).getVariable() != null && iterator.equals(((Attribute) setClassifierRole.getAttributes().get(i)).getVariable().getName())){
					iteratorVariable = ((Attribute) setClassifierRole.getAttributes().get(i)).getVariable();
					break;
				}
			}
			if(iteratorVariable != null){
				VariableDeclaration sumVariable = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
				sumVariable.setName(sum);
				sumVariable.setType(iteratorVariable.getType());
				setClassifierRole.setSum(sumVariable);
				setClassifierRole.setIterator(iteratorVariable);
				VOCLEditorPlugin.getConstraint( setClassifierRole ).getAllVariables().add(sumVariable);
			}
		}		
		if (loc != null) {
			setClassifierRole.setX(loc.x);
			setClassifierRole.setY(loc.y);
		}
	}	
	
	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		parent.getClassifierRoles().remove(setClassifierRole);
		for(int i=0; i<setClassifierRole.getAttributes().size(); i++){
			if(((Attribute) setClassifierRole.getAttributes().get(i)).getVariable() != null)
				VOCLEditorPlugin.getConstraint( parent ).getAllVariables().remove(((Attribute) setClassifierRole.getAttributes().get(i)).getVariable());
		}			
		if(setClassifierRole.getSize() != null)
			VOCLEditorPlugin.getConstraint( parent ).getAllVariables().remove(setClassifierRole.getSize());
		if(setClassifierRole.getSum() != null)
			VOCLEditorPlugin.getConstraint( parent ).getAllVariables().remove(setClassifierRole.getSum());
	}
	
	/**
	 * Returns the parent nav exp.
	 * @return the parent nav exp
	 */
	public NavExp getParent() {
		return parent;
	}
	
	/**
	 * Sets the parent nav exp.
	 * @param newParent the new parent nav exp to set
	 */
	public void setParent(NavExp newParent) {
		parent = newParent;
	}
	
	/**
	 * Returns the set classifier role.
	 * @return the set classifier role
	 */
	public SetClassifierRole getSetClassifierRole() {
		return setClassifierRole;
	}
	
	/**
	 * Sets the set classifier role.
	 * @param setClassifierRole the set classifier role to set
	 */
	public void setSetClassifierRole(SetClassifierRole setClassifierRole ) {
		this.setClassifierRole = setClassifierRole;
	}	
	
	/**
	 * Sets the location where the set classifier role located.
	 * @param loc the location
	 */
	public void setLocation (Point loc) {
		this.loc = loc;
	}
	
	/**
	 * Sets the name.
	 * @param name the name
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Sets the type.
	 * @param type the type
	 */
	public void setType(String type){
		this.type = type;
	}
	
	/**
	 * Sets the operation kind.
	 * @param opKind the operation kind
	 */
	public void setOpKind(String opKind){
		this.opKind = opKind;
	}
	
	
	/**
	 * Sets the size variable name.
	 * @param size the size variable name
	 */
	public void setSizeVariable(String size){
		this.size = size;
	}
	
	/**
	 * Sets the iterator variable name.
	 * @param iterator the iterator variable name
	 */
	public void setIteratorVariable(String iterator){
		this.iterator = iterator;
	}
	
	/**
	 * Sets the sum variable name.
	 * @param sum the sum variable name
	 */
	public void setSumVariable(String sum){
		this.sum = sum;
	}
	
	/**
	 * Sets the list of attributes.
	 * @param attributes the list of attributes
	 */
	public void setAttributes(List attributes){
		this.attributes = attributes;		
	}
	
	/**
	 * Sets the command attributes.
	 * @param name the name 
	 * @param type the type
	 * @param opKind the operation kind
	 * @param size the size variable
	 * @param attributes the list of attributes
	 */
	public void setCommandAttributes(String name, String type, String opKind, String size, String iterator, String sum, List attributes) {
		setName(name);
		setType(type);
		setOpKind(opKind);
		setSizeVariable(size);
		setIteratorVariable(iterator);
		setSumVariable(sum);
		setAttributes(attributes);		
	}
}
