/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;



import java.util.List;
import java.util.Vector;

import vocleditor.VOCLEditorPlugin;
import VOCLModel.Attribute;
import VOCLModel.ClassifierRole;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.NavExp;
import VOCLModel.SetClassifierRole;
import VOCLModel.VariableDeclaration;


/**
 * Command for deleting a place.
 */
public class DeleteAttributeCommand extends org.eclipse.gef.commands.Command {
	private	Attribute attribute;
	private ClassifierRole parent;
	private VariableDeclaration iteratorVariable;
	private VariableDeclaration sumVariable;
	private List deleteConditionCommands = new Vector();
	private ClearLetExpsSubCommand clearLetexpsCommand = null;
	
	/**
	 * Creates a new DeleteAttributeCommand.
	 *
	 */
	public DeleteAttributeCommand() {
		setLabel("delete attribute");
	}

	
	/**
	 * Sets the Attribute to be deleted
	 * @param attr Attribute to be deleted
	 */
	public void setAttribute( Attribute attr )
	{
		this.attribute = attr;
	}

	/**
	 * Sets the ClassifierRole the Attribute is to be deleted from
	 * @param parent ClassifierRole which owns the Attribute to be deleted
	 */	
	public void setParent( ClassifierRole parent) {
		this.parent = parent;
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() {		
		parent.getAttributes().remove(attribute);
		if(attribute.getVariable() != null){
			if(parent instanceof SetClassifierRole && "sum".equals(((SetClassifierRole) parent).getOpKind())){
				if(attribute.getVariable().equals(((SetClassifierRole) parent).getIterator())){
					iteratorVariable = attribute.getVariable();
					sumVariable = ((SetClassifierRole) parent).getSum();
					((SetClassifierRole) parent).setOpKind("");
					((SetClassifierRole) parent).setIterator(null);
					((SetClassifierRole) parent).setSum(null);
				}
			}			
			
			Constraint constraint = VOCLEditorPlugin.getConstraint(parent);
			List conditions = VOCLEditorPlugin.getAllConditions(constraint);
			for(int i=0; i<conditions.size(); i++){
				Condition condition = (Condition) conditions.get(i);
				VariableDeclaration left = condition.getLeft();
				VariableDeclaration right = condition.getRight();
				if(left.equals(attribute.getVariable()) || ((right!=null) && (right.equals(attribute.getVariable())))
						|| left.equals(sumVariable) || ((right!=null) && (right.equals(sumVariable)))){
					DeleteConditionCommand deleteConditionCommand = new DeleteConditionCommand();
					deleteConditionCommand.setCondition(condition);
					deleteConditionCommand.setParent((NavExp) condition.eContainer());
					deleteConditionCommand.execute();
					deleteConditionCommands.add(deleteConditionCommand);
				}
			}
			constraint.getAllVariables().remove(attribute.getVariable());
			if(sumVariable != null)
				constraint.getAllVariables().remove(sumVariable);

			// eventually clear affected letExps
			clearLetexpsCommand = new ClearLetExpsSubCommand( parent.getNavExp() );
			clearLetexpsCommand.execute();
		}
		
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		parent.getAttributes().add( attribute );
		if(attribute.getVariable() != null)
			VOCLEditorPlugin.getConstraint( parent ).getAllVariables().add(attribute.getVariable());		
		if(parent instanceof SetClassifierRole && iteratorVariable != null && sumVariable != null){			
			VOCLEditorPlugin.getConstraint( parent ).getAllVariables().add(sumVariable);
			((SetClassifierRole) parent).setOpKind("sum");
			((SetClassifierRole) parent).setIterator(iteratorVariable);
			((SetClassifierRole) parent).setSum(sumVariable);
		}
		
		for(int i=0; i<deleteConditionCommands.size(); i++){
			DeleteConditionCommand deleteConditionCommand = (DeleteConditionCommand) deleteConditionCommands.get(i);
			deleteConditionCommand.undo();
		}
		clearLetexpsCommand.undo();
	}
	
}
