/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import org.eclipse.gef.commands.Command;
import VOCLModel.*;

/**
 * @author manaljav
 * Command class for edit the condition properties.
 */
public class EditConditionCommand extends Command {
	private Condition condition;
	private String oldOp, oldValue, newOp, newValue;
    private VariableDeclaration oldLeft, oldRight, newLeft, newRight;
    private boolean newLeftPre, oldLeftPre, newRightPre, oldRightPre;
    
    /**
     * The constructor.
     */
	public EditConditionCommand(){
		setLabel("edit condition");
	}
	
	/**
	 * Sets the condition.
	 * @param condition the condition
	 */
	public void setCondition(Condition condition) {
		this.condition = condition;
		oldOp = condition.getOp();
		oldValue = condition.getValue();
		oldLeft = condition.getLeft();
		oldRight = condition.getRight();
		oldLeftPre = condition.isLeftPre();
		oldRightPre = condition.isRightPre();
	}

	/**
	 * Executes the command.
	 */
	public void execute() {
		condition.setOp(newOp);
		condition.setValue(newValue);
		condition.setLeft(newLeft);
		condition.setRight(newRight);
		condition.setLeftPre(newLeftPre);
		condition.setRightPre(newRightPre);
	}
	
	/** 
	 * Undo command execution.
	 */
	public void undo() {
		condition.setOp(oldOp);
		condition.setValue(oldValue);
		condition.setLeft(oldLeft);
		condition.setRight(oldRight);
		condition.setLeftPre(oldLeftPre);
		condition.setRightPre(oldRightPre);
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo(){
		execute();
	}
	
	/**
	 * Sets the operation.
	 * @param op the operation set
	 */
	public void setOp(String op) {
		this.newOp = op;
	}
	
	/**
	 * Sets the value.
	 * @param value the value to set
	 */
	public void setValue(String value) {
		this.newValue = value;
	}
	
	/**
	 * Sets the right variable.
	 * @param right the right variable
	 */
	public void setRight(VariableDeclaration right) {
		this.newRight = right;
	}
	
	/**
	 * Sets the left variable.
	 * @param left the left variable
	 */
	public void setLeft(VariableDeclaration left) {
		this.newLeft = left;
	}
	
	/**
	 * Sets the left pre.
	 * @param leftPre the left pre
	 */
	public void setLeftPre(boolean leftPre){
		this.newLeftPre = leftPre;		
	}
	
	/**
	 * Sets the rigth pre.
	 * @param rightPre the right pre
	 */
	public void setRightPre(boolean rightPre){
		this.newRightPre = rightPre;
	}
	
	/**
	 * Sets the command attributes.
	 * @param op the operation.
	 * @param value the value
	 * @param left the left variable
	 * @param right the right variable
	 */
	public void setCommandAttributes(String op, String value, VariableDeclaration left, VariableDeclaration right, boolean leftPre, boolean rightPre) {
		setOp(op);
		setValue(value);
		setRight(right);
		setLeft(left);
		setLeftPre(leftPre);
		setRightPre(rightPre);
	}	
}
