/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import java.util.List;
import java.util.Vector;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.commands.Command;

import vocleditor.VOCLEditorPlugin;
import vocleditor.model.ModelCreationFactory;
import VOCLModel.*;

/**
 * Command class for edit the context properties.
 * @author manaljav, bjoern
 */
public class EditContextCommand extends Command {
	private Context context;
	private String oldContextName;
	private String oldConstraintName, newConstraintName;
	private String oldType, newType;
    private ClassifierRole oldClassifier, newClassifier;
    private Method oldMethod, newMethod;
    private boolean oldIsClass, newIsClass;
    private VariableDeclaration oldResultVar = null;
    private VariableDeclaration newResultVar = null;
	private List deleteConditionCommands = new Vector();

    /**
     * The constructor.
     */
	public EditContextCommand( Context context ) {
		setLabel("edit condition");
		this.context = context;
		
		oldContextName       = context.getName();
		oldConstraintName    = context.getConstraint().getName(); 
		oldType       = context.getType();
		oldIsClass    = context.isIsClassifier();
		oldClassifier = context.getContextClassifier();
		oldMethod     = context.getContextMethod();
		
		EList allVariables = context.getConstraint().getAllVariables(); 
		for( int i = 0; i < allVariables.size(); i++ ) {
			if( "result".equals( ((VariableDeclaration) allVariables.get(i)).getName() )) {
				oldResultVar = ((VariableDeclaration) allVariables.get(i));
				break;
			}	
		}
	}
	
	/**
	 * Executes the command.
	 */
	public void execute() {
		context.setIsClassifier( newIsClass );
		context.setName( "" );  // is done in ContextEditPart after model changes
		context.setType( newType );
		context.setContextClassifier( newClassifier );
		context.setContextMethod( newMethod );
		context.getConstraint().setName( newConstraintName );
		context.getConstraint().setKind( newType );	
		if (newIsClass &&  newClassifier!=null) newClassifier.setContext(context);
		else if (newMethod!=null) newMethod.setContext(context);

		if ( oldResultVar != null )
			context.getConstraint().getAllVariables().remove( oldResultVar );
		if ( newResultVar != null ) {
			context.getConstraint().getAllVariables().add( newResultVar );
			newResultVar.setType( newMethod.getReturnType() );
		}
		
		if ( oldResultVar != null && newResultVar == null ) {
			List conditions = VOCLEditorPlugin.getAllConditions( context.getConstraint() );
			for( int i = 0; i < conditions.size(); i++ ) {
				Condition condition = (Condition) conditions.get(i);
				if( condition.getLeft() == oldResultVar || condition.getRight() == oldResultVar ) {
					DeleteConditionCommand deleteConditionCommand = new DeleteConditionCommand();
					deleteConditionCommand.setCondition( condition );
					deleteConditionCommand.setParent( (NavExp) condition.eContainer());
					deleteConditionCommand.execute();
					deleteConditionCommands.add(deleteConditionCommand);
				}
			}
		}
	}
	
	/** 
	 * Undo command execution.
	 */
	public void undo() {
		String contextName = "";
		context.setIsClassifier( oldIsClass );
		context.setName( oldContextName );
		context.setType( oldType );
		context.setContextClassifier( oldClassifier );
		context.setContextMethod( oldMethod );
		context.getConstraint().setName( oldConstraintName );
		context.getConstraint().setKind( oldType );
		if (oldIsClass && oldClassifier!=null) oldClassifier.setContext(context);
		else if (oldMethod!=null) oldMethod.setContext(context);
		
		if ( newResultVar != null )
			context.getConstraint().getAllVariables().remove( newResultVar );
		if ( oldResultVar != null ) {
			context.getConstraint().getAllVariables().add( oldResultVar );
			oldResultVar.setType( oldMethod.getReturnType() );
		}
		
		for(int i=0; i<deleteConditionCommands.size(); i++){
			DeleteConditionCommand deleteConditionCommand = (DeleteConditionCommand) deleteConditionCommands.get(i);
			deleteConditionCommand.undo();
		}

	}
	
	/**
	 * Redo command execution.
	 */
	public void redo(){
		execute();
	}
	

	/**
	 * Sets the command attributes.
	 * @param constraintName the new name of the constraint.
	 * @param type the type
	 * @param isClassifier indicating whether the Constraint uses a classifier or not
	 * @param classifier the {@link ClassifierRole} if necessary.
	 * @param method The referenced {@link Method}
	 */
	public void setCommandAttributes(String constraintName, String type, boolean isClassifier, ClassifierRole classifier, Method method ) {
		newConstraintName = constraintName;
		newType      	   = type;
		newIsClass        = isClassifier;
		newClassifier     = classifier;
		newMethod         = method;
		if ( newMethod != null && newMethod.getReturnVariable() == null ) {
			if ( oldResultVar != null )
				newResultVar = oldResultVar;
			else {
				newResultVar = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
				newResultVar.setName( "result" );
			}
		}		
	}	
}
