/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import org.eclipse.gef.commands.Command;

import VOCLModel.LetExp;
import VOCLModel.VariableDeclaration;

/**
 * Command class for edit the properties of a boolean expression.
 * @author bjoern
 */
public class EditLetExpCommand extends Command {
	private LetExp letexp;
	private boolean oldHorizontal, newHorizontal;
	private int oldSplitPercent, newSplitPercent;
	private VariableDeclaration oldVar, newVar;
	private boolean isNegated, wasNegated;

	/**
	 * creates a new EditLetExpCommand
	 * @param letexp the letexp to be altered
	 */
	public EditLetExpCommand( LetExp letexp )
	{
		setLabel("edit attribute");
		this.letexp = letexp;
		this.oldHorizontal = letexp.isSplitHorizontal();
		this.oldSplitPercent = letexp.getSplitPercent();
		this.oldVar = letexp.getLetVar();
		this.wasNegated = letexp.isNegated();
	}

	/**
	 * sets the new letexp attributes to be set in this commmand
	 * @param isHorizontal the new split horizontal flag
	 * @param splitPercentage the split percentage
	 * @param varDecl the {@link VariableDeclaration} of this {@link LetExp}
	 * @param isNegated indicating whether to negate the expression
	 */
	public void setCommandAttributes( boolean isHorizontal, int splitPercentage, VariableDeclaration varDecl, boolean isNegated )
	{
		this.newSplitPercent = splitPercentage;
		this.newHorizontal = isHorizontal;
		this.newVar = varDecl;
		this.isNegated = isNegated;
	}
	
	/**
	 * execute this command
	 */
	public void execute() {
		letexp.setSplitHorizontal( newHorizontal );
		letexp.setSplitPercent( newSplitPercent );
		letexp.setLetVar( newVar );
		letexp.setNegated( isNegated );
	}
	
	/** 
	 * Undo command execution.
	 */
	public void undo() {
		letexp.setSplitHorizontal( oldHorizontal );
		letexp.setSplitPercent( oldSplitPercent );
		letexp.setLetVar( oldVar );
		letexp.setNegated( wasNegated );
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo(){
		execute();
	}	
}
