/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;

import org.eclipse.gef.commands.Command;

import vocleditor.VOCLEditorPlugin;
import vocleditor.model.ModelCreationFactory;

import VOCLModel.Attribute;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.NavExp;
import VOCLModel.SetClassifierRole;
import VOCLModel.VariableDeclaration;



/**
 * @author tugstugi
 * Command class for edit the set classifier role properties.
 */
public class EditSetClassifierRoleCommand extends Command {	
	private SetClassifierRole setClassifierRole;
	private String name_old = "";
	private String name_new = "";
	private String type_old = "";
	private String type_new = "";
	private String opKind_old = "";
	private String opKind_new = "";
	private String sizeName_old = "";
	private String sizeName_new = "";
	private String iteratorName_old = "";
	private String iteratorName_new = "";
	private String sumName_old = "";
	private String sumName_new = "";
	private VariableDeclaration size_new = null;
	private VariableDeclaration size_old = null;
	private VariableDeclaration iterator_new = null;
	private VariableDeclaration iterator_old = null;
	private VariableDeclaration sum_new = null;
	private VariableDeclaration sum_old = null;
	private List  attributes_old = new ArrayList();
	private List  attributes_new = new ArrayList();
	private Hashtable changedVariables = new Hashtable();
	private Hashtable initialOldVariables = new Hashtable();
	private Hashtable initialNewVariables = new Hashtable();
	
	private Constraint constraint;
	private List deleteConditionCommands = new Vector();

	/**
	 * The constructor.
	 */
	public EditSetClassifierRoleCommand(){
		setLabel("edit classifier role");
	}
	
	/**
	 * Sets the set classifier role.
	 * @param setClassifierRole the set classifier role to set.
	 */
	public void setClassifierRole(SetClassifierRole setClassifierRole) {
		this.setClassifierRole = setClassifierRole;
		name_old = setClassifierRole.getName();
		type_old = setClassifierRole.getType();
		opKind_old = setClassifierRole.getOpKind();
		sizeName_old = setClassifierRole.getSize() != null ? setClassifierRole.getSize().getName() : "";
		size_old = setClassifierRole.getSize();
		iteratorName_old = setClassifierRole.getIterator() != null ? setClassifierRole.getIterator().getName() : "";
		iterator_old = setClassifierRole.getIterator();
		sumName_old = setClassifierRole.getSum() != null ? setClassifierRole.getSum().getName() : "";
		sum_old = setClassifierRole.getSum();
		for(int i=0; i<setClassifierRole.getAttributes().size(); i++){
			Attribute attribute = (Attribute) setClassifierRole.getAttributes().get(i);
			attributes_old.add(attribute);
			if(attribute.getVariable() != null)
				initialOldVariables.put(attribute, attribute.getVariable());
		}
		constraint = VOCLEditorPlugin.getConstraint(setClassifierRole); 
	}

	/**
	 * Entry point for command execution.
	 */
	public void execute() {
		List deletedVariables = new Vector();
		
		setClassifierRole.setName(name_new);
		setClassifierRole.setType(type_new);
		setClassifierRole.setOpKind(opKind_new);		
		if(size_new != null){
			if(size_old != null){				
				size_old.setName(sizeName_new);
				setClassifierRole.setSize(size_old);
			}else{				
				setClassifierRole.setSize(size_new);
				constraint.getAllVariables().add(size_new);
			}
		}else{
			if(size_old != null){
				setClassifierRole.setSize(null);
				constraint.getAllVariables().remove(size_old);
				deletedVariables.add(size_old);
			}
		}		
		
		for (int i=0; i<attributes_old.size(); i++){
			Attribute attribute = (Attribute) attributes_old.get(i);
			if(!attributes_new.contains(attribute)){
				setClassifierRole.getAttributes().remove(attribute);
				if(constraint.getAllVariables().contains(attribute.getVariable())){
					VariableDeclaration oldVariable = (attribute).getVariable();
					constraint.getAllVariables().remove(oldVariable);
					deletedVariables.add(oldVariable);
					oldVariable.getAttributes().remove(attribute);
				}
			}
		}
		
		for (int i=0; i<attributes_new.size(); i++){
			Attribute attribute = (Attribute) attributes_new.get(i);
			if(! attributes_old.contains(attribute)){
				if(initialNewVariables.containsKey(attribute))
					attribute.setVariable((VariableDeclaration) initialNewVariables.get(attribute));
				setClassifierRole.getAttributes().add(attribute);
				if(attribute.getVariable() != null && !constraint.getAllVariables().contains(attribute.getVariable())){
					VariableDeclaration newVariable = attribute.getVariable();
					if(changedVariables.containsKey(newVariable)){
						VariableDeclaration oldVariable = (VariableDeclaration) changedVariables.get(newVariable);
						String oldName = oldVariable.getName();
						String oldType = oldVariable.getType();
						oldVariable.setName(newVariable.getName());
						oldVariable.setType(newVariable.getType());
						newVariable.setName(oldName);
						newVariable.setType(oldType);
						attribute.setVariable(oldVariable);
						constraint.getAllVariables().add(oldVariable);
						deletedVariables.remove(oldVariable);
					}else{				
						attribute.setVariable(newVariable);
						constraint.getAllVariables().add(newVariable);
					}
				}				
			}
		}
		
		setClassifierRole.setIterator(iterator_new);
		if(iterator_new != null){
			if(sum_old != null){				
				sum_old.setName(sumName_new);
				sum_old.setType(iterator_new.getType());
				setClassifierRole.setSum(sum_old);
			}else{		
				sum_new.setName(sumName_new);
				sum_new.setType(iterator_new.getType());
				setClassifierRole.setSum(sum_new);
				constraint.getAllVariables().add(sum_new);
			}			
		}else{
			if(sum_old != null){
				constraint.getAllVariables().remove(sum_old);
				deletedVariables.add(sum_old);
				setClassifierRole.setSum(null);
			}
		}
		
		List conditions = VOCLEditorPlugin.getAllConditions(constraint);
		for(int i=0; i<conditions.size(); i++){
			Condition condition = (Condition) conditions.get(i);
			if(condition.getLeft() != null && deletedVariables.contains(condition.getLeft())
					|| condition.getRight() != null && deletedVariables.contains(condition.getRight())){
				DeleteConditionCommand deleteConditionCommand = new DeleteConditionCommand();
				deleteConditionCommand.setCondition(condition);
				deleteConditionCommand.setParent((NavExp) condition.eContainer());
				deleteConditionCommand.execute();
				deleteConditionCommands.add(deleteConditionCommand);
			}				
		}
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		setClassifierRole.setName(name_old);
		setClassifierRole.setType(type_old);
		setClassifierRole.setOpKind(opKind_old);
		if(size_old != null){
			size_old.setName(sizeName_old);
			setClassifierRole.setSize(size_old);
			if(! constraint.getAllVariables().contains(size_old))
				constraint.getAllVariables().add(size_old);
		}else{
			if(size_new != null){
				setClassifierRole.setSize(null);
				constraint.getAllVariables().remove(size_new);
			}
		}	
		
		for(int i=0; i<attributes_new.size(); i++){
			Attribute attribute = (Attribute) attributes_new.get(i);
			if(! attributes_old.contains(attribute)){
				setClassifierRole.getAttributes().remove(attribute);
				if(constraint.getAllVariables().contains(attribute.getVariable())){
					VariableDeclaration newVariable = (attribute).getVariable();
					constraint.getAllVariables().remove(newVariable);
					newVariable.getAttributes().remove(attribute);
				}
			}
		}
		
		for(int i=0; i<attributes_old.size(); i++){
			Attribute attribute = (Attribute) attributes_old.get(i);
			if(!attributes_new.contains(attribute)){			
				if(initialOldVariables.containsKey(attribute))
					attribute.setVariable((VariableDeclaration) initialOldVariables.get(attribute));
				setClassifierRole.getAttributes().add(attribute);
				if(attribute.getVariable() != null && !constraint.getAllVariables().contains(attribute.getVariable())){
					VariableDeclaration oldVariable = (attribute).getVariable();
					if(changedVariables.containsValue(oldVariable)){
						VariableDeclaration newVariable = null;
						Enumeration enumeration = changedVariables.keys();
						while(enumeration.hasMoreElements()){
							Object key = enumeration.nextElement();
							if(oldVariable.equals(changedVariables.get(key))){
								newVariable = (VariableDeclaration) key;
								break;
							}								
						}
						String newName = oldVariable.getName();
						String newType = oldVariable.getType();
						oldVariable.setName(newVariable.getName());
						oldVariable.setType(newVariable.getType());
						newVariable.setName(newName);
						newVariable.setType(newType);
						attribute.setVariable(oldVariable);
						attribute.setVariable(oldVariable);
						constraint.getAllVariables().add(oldVariable);
					}else{				
						attribute.setVariable(oldVariable);
						constraint.getAllVariables().add(oldVariable);
					}
				}
			}
		}
		
		setClassifierRole.setIterator(iterator_old);
		if(iterator_old != null){
			sum_old.setName(sumName_old);
			sum_old.setType(iterator_old.getType());
			setClassifierRole.setSum(sum_old);
			if(! constraint.getAllVariables().contains(sum_old))
				constraint.getAllVariables().add(sum_old);
		}else{
			if(sum_new != null){
				setClassifierRole.setSum(null);
				constraint.getAllVariables().remove(sum_new);
			}
		}
		
		for(int i=0; i<deleteConditionCommands.size(); i++){
			DeleteConditionCommand deleteConditionCommand = (DeleteConditionCommand) deleteConditionCommands.get(i);
			deleteConditionCommand.undo();
		}
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo(){
		execute();
	}	
	
	/**
	 * Sets the new name.
	 * @param name the new name
	 */
	public void setName(String name) {
		this.name_new = name;
	}
	
	/**
	 * Sets the new type.
	 * @param type value the new type
	 */
	public void setType(String type){
		this.type_new = type;
	}
	
	/**
	 * Sets the new operation kind.
	 * @param opKind the new operation kind
	 */
	public void setOpKind(String opKind){
		this.opKind_new = opKind;
		if("size".equals(opKind_new)){
			size_new = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
			size_new.setName(sizeName_new);
			size_new.setType("int");
		}else if("sum".equals(opKind_new)){
			sum_new = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
			sum_new.setName(sizeName_new);
			sum_new.setType("int");
		}else{
			iterator_new = null;			
		}
	}
	
	/**
	 * Sets the name of the size variable.
	 * @param size the size variable name
	 */
	public void setSizeVariableName(String size){
		this.sizeName_new = size;
	}
	
	/**
	 * Sets the name of the iterator variable.
	 * @param iterator the iterator variable name
	 */
	public void setIteratorVariableName(String iterator){
		this.iteratorName_new = iterator;
		for(int i=0; i<attributes_new.size(); i++){
			Attribute attribute = (Attribute) attributes_new.get(i);
			if(attribute.getVariable() != null && attribute.getVariable().getName().equals(iteratorName_new)){
				iterator_new = attribute.getVariable();				
				if(changedVariables.containsKey(iterator_new))
					iterator_new = (VariableDeclaration) changedVariables.get(iterator_new);
				return;
			}
		}
	}
	
	/**
	 * Sets the name of the sum variable.
	 * @param sum the sum variable name
	 */
	public void setSumVariableName(String sum){
		this.sumName_new = sum;
	}
	
	/**
	 * Sets the the hashtable of changed variables.
	 * @param changedVariables the hashtable of changed variables.
	 */
	public void setChangedVariables(Hashtable changedVariables){
		this.changedVariables = changedVariables;
	}
	
	/**
	 * Sets the new list of attributes.
	 * @param attributes the new list of attributes
	 */
	public void setAttributes(List attributes){
		for(int i=0; i<attributes.size(); i++){
			Attribute attribute = (Attribute) attributes.get(i);
			attributes_new.add(attribute);
			if(attribute.getVariable() != null)
				initialNewVariables.put(attribute, attribute.getVariable());
		}	
	}
	
	/**
	 * Sets the command attributes.
	 * @param name the new name
	 * @param type the new type
	 * @param opKind the new operation kind
	 * @param attributes the new list of attributes
	 */
	public void setCommandAttributes(String name, String type, String opKind, String sizeName, String iteratorName, String sumName, Hashtable changedVariables, List attributes) {
		setName(name);
		setType(type);
		setSizeVariableName(sizeName);			
		setChangedVariables(changedVariables);
		setAttributes(attributes);		
		setIteratorVariableName(iteratorName);
		setSumVariableName(sumName);
		setOpKind(opKind);
	}
}
