/*
 * Created on 20.06.2005
 *
 */
package vocleditor.commands;

import org.eclipse.gef.commands.Command;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;

import VOCLModel.AssociationRole;

/**
 * Command for moving AssociationRole Labels
 * @author jan
 *
 */
public class MoveAssociationRoleLabelCommand extends Command {
	/**
	 * The associationrole that's label is to be moved
	 */
	AssociationRole associationRole = null;
	
	/**
	 * The curent offset
	 */
	Point offset = null;
	
	/**
	 * The label's figure
	 */
	IFigure figure = null;
	
	/**
	 * Old offset for undo
	 */
	Point oldOffset = new Point();
	
	/**
	 * Indicating whether there was a custom position before
	 */
	boolean wasCustomPosition;
	
	/**
	 * The endpoint of the connection. Used to calculate offset if the label was automatically placed before.
	 */
	Point connectionEndPoint;

	/**
	 * Constructor
	 * @param associationRole The AssociationRole which's label is to be moved
	 * @param figure The label's figure
	 * @param offset The offset of movement
	 * @param connectionEndPoint Endpoint of the connection.
	 */
	public MoveAssociationRoleLabelCommand(AssociationRole associationRole, IFigure figure, Point offset, Point connectionEndPoint) {
		this.associationRole = associationRole;
		this.figure = figure;
		this.offset = offset;
		setLabel("Move associationrole label");
		this.connectionEndPoint = connectionEndPoint;
	}

	/**
	 * @see Command#execute()
	 */
	public void execute()
	{
		if(associationRole.getLabelDecoration().isCustomPosition())
			oldOffset = new Point(associationRole.getLabelDecoration().getX(), associationRole.getLabelDecoration().getY());
		else {
			oldOffset = figure.getBounds().getLocation().getCopy();
			oldOffset.translate(-connectionEndPoint.x, -connectionEndPoint.y);
		}
		wasCustomPosition = associationRole.getLabelDecoration().isCustomPosition();
		Point newOffset = oldOffset.getCopy();
		newOffset.translate(offset);
		associationRole.getLabelDecoration().setX(newOffset.x);
		associationRole.getLabelDecoration().setY(newOffset.y);
		associationRole.getLabelDecoration().setCustomPosition(true);
		associationRole.setName(associationRole.getName()); // Just to have the associationrole redrawn.
	}

	/**
	 * @see Command#redo()
	 */
	public void redo()
	{
		execute();
	}

	/**
	 * @see Command#undo()
	 */
	public void undo()
	{
		associationRole.getLabelDecoration().setX(oldOffset.x);
		associationRole.getLabelDecoration().setY(oldOffset.y);
		associationRole.getLabelDecoration().setCustomPosition(wasCustomPosition);
		associationRole.setName(associationRole.getName()); // Just to have the associationrole redrawn.
	}
}
