
/**
 * VOCL --> OCL Converter with AGG
 */
package vocleditor.converter;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import agg.attribute.impl.ValueTuple;
import agg.convert.ConverterXML;
import agg.xt_basis.BaseFactory;
import agg.xt_basis.GraGra;
import agg.xt_basis.GraTraEvent;
import agg.xt_basis.GraTraEventListener;
import agg.xt_basis.LayeredGraTraImpl;
import agg.xt_basis.Node;

import vocleditor.VOCLEditorPlugin;

public class Converter implements GraTraEventListener {	
	// The main AGG Graph Grammar
	private File voclFile;
	private GraGra gragra;
	private String oclString;
	
	public Converter(File voclFile){
		this.voclFile = voclFile;
		oclString = "";
	}
	
	
	/**
	 * Listener to AGG Events
	 */
	public void graTraEventOccurred(GraTraEvent event){
		if (event.getMessage() == GraTraEvent.TRANSFORM_FINISHED) {
			VOCLEditorPlugin.printDebug("OCLStringbefore"+ oclString);
		   Enumeration nodes = gragra.getGraph().getNodes();
		   while(nodes.hasMoreElements()) {  
		   	   Node node = (Node) nodes.nextElement();
		   	   VOCLEditorPlugin.printDebug(node.getType().getStringRepr());
		       if(node.getType().getStringRepr().equals("OCLString")) { 
		       	    ValueTuple attrInstance= (ValueTuple) node.getAttribute();
		       	    String tmp = (String) attrInstance.getValueAt("text");
		       	    while ( tmp.contains( "\\n" ))
		       	    	tmp = tmp.replace("\\n","\n");
		       	    oclString=tmp;
		       	    
		       	  /*  Pattern p=Pattern.compile("and");
		       	    String[] result=p.split(tmp);
		       	    for (int i=0;i<result.length;i++) {
		       	    	if (i==result.length-1)
		       	    	oclString+=result[i];
		       	    	else if (i==0) {
		       	    	oclString+=(" "+result[i]+" and\n");
		       	    	}
		       	    	else
		       	    	oclString+=(result[i]+" and\n");
		       	    }*/
		         }
		     }
		 }
	}
	
	/**
	 * Imports a VOCL-GXL-Graph in a AGG GraphGrammar
	 * and transforms it to an OCL String
	 * @throws IOException 
	 * @throws TransformerException 
	 */
	public void convert() throws IOException, TransformerException {
		// XSLT convert
//		File tempDirectory = new File("c:/programme/agg/TestAusbaustufe3","test.tmp").getParentFile();
		File tempDirectory = File.createTempFile("test", ".tmp").getParentFile();
		TransformerFactory tf = TransformerFactory.newInstance();
		Transformer transformer = tf.newTransformer(new StreamSource(Converter.class.getResourceAsStream("vocl2gxl.xsl")));
		File resultGXLFile = createFile(tempDirectory, "result.gxl");
		transformer.transform(new StreamSource(new FileInputStream(voclFile)), new StreamResult(new FileOutputStream(resultGXLFile)));
		// Konvertierungsdateien werden in TEMP Verzeichnis kopiert!
		File voclTypGraphGGXFile = createFile(tempDirectory, "VOCLTypGraph3.ggx");
		copy(Converter.class.getResourceAsStream("VOCLTypGraph3.ggx"), new FileOutputStream(voclTypGraphGGXFile));
		File gxl2ggxXSLFile = createFile(tempDirectory, "gxl2ggx.xsl");
		copy(Converter.class.getResourceAsStream("gxl2ggx.xsl"), new FileOutputStream(gxl2ggxXSLFile));
		File agglayoutDTDFile = createFile(tempDirectory, "agglayout.dtd");
		copy(Converter.class.getResourceAsStream("agglayout.dtd"), new FileOutputStream(agglayoutDTDFile));
		File gxlDTDFile = createFile(tempDirectory, "gxl.dtd");
		copy(Converter.class.getResourceAsStream("gxl.dtd"), new FileOutputStream(gxlDTDFile));
		File gtsDTDFile = createFile(tempDirectory, "gts.dtd");
		copy(Converter.class.getResourceAsStream("gts.dtd"), new FileOutputStream(gtsDTDFile));
		// Get a base factory BaseFactory 
		BaseFactory bf = BaseFactory.theFactory(); 
		// Load graph grammar 
		gragra = bf.createGraGra();
		gragra.load(voclTypGraphGGXFile.getAbsolutePath());
		// Load GXL graph and set it as start graph
		ConverterXML converter = new ConverterXML();
		// files: agglayout.dtd, gxl2ggx.xsl, gxl.dtd, gts.dtd
		// have to be present in converting directory
		File resultGGXFile = createFile(tempDirectory, "result_gxl.ggx");
		converter.gxl2ggx(resultGXLFile.getAbsolutePath(), resultGGXFile.getAbsolutePath(), gxl2ggxXSLFile.getAbsolutePath());
		VOCLEditorPlugin.printDebug("Finished");
		GraGra hgragra = bf.createGraGra();
		hgragra.load(resultGGXFile.getAbsolutePath());
		gragra.importGraph(hgragra.getGraph());
		// Set up layered graph transformation
		LayeredGraTraImpl gratra = new LayeredGraTraImpl(); 
		gratra.setGraGra(gragra);
		gratra.setHostGraph(gragra.getGraph());
		gratra.addGraTraListener(this); 
		gratra.setGraTraOptions(gragra.getGraTraOptions());
		gratra.transform();		
		
		resultGXLFile.delete();
		voclTypGraphGGXFile.delete();
		gxl2ggxXSLFile.delete();
		agglayoutDTDFile.delete();
		gxlDTDFile.delete();
		gtsDTDFile.delete();
		resultGGXFile.delete();
	}
	
	/**
	 * Creates a temporary file.
	 * @param tempDirectory temporary directory 
	 * @param name name of the temporary file
	 * @return the temporary file 
	 * @throws IOException
	 */
	private File createFile(File tempDirectory, String name) throws IOException{
		File file = new File(tempDirectory, name);
		if(file.exists())
			file.delete();
		file.createNewFile();
		return file;
	}
	
	/**
	 * Copy from the input stream to the output stream.
	 * @param inputStream the input stream
	 * @param outputStream the input stream
	 * @throws IOException
	 */
	private void copy(InputStream inputStream, OutputStream outputStream) throws IOException{
		byte buffer[] = new byte[1024];
		int read = 0;
		try{
			while(true){
				read = inputStream.read(buffer);
				if(read == -1)
					break;
				outputStream.write(buffer, 0, read);
			}
		}finally{
			if(inputStream != null){
				try{
					inputStream.close();
				}finally{
					if(outputStream != null)
						outputStream.close();
				}
			}
		}
	}
	
	public String getOCLString(){
		return oclString;
	}
}
