/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.dialog;

import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.jface.dialogs.IDialogConstants;

import VOCLModel.AssociationRole;
import VOCLModel.ClassifierRole;

/**
 *
 * JFace based dialog used to edit AssociationRole properties 
 * <br>
 * @author bjoern, stefan, jan
 */
public class AssociationPropertiesDialog extends TitleAreaDialog implements SelectionListener,ModifyListener
{
	String name;
	Text nameText;
	String nameSourceClassifier;
	String nameTargetClassifier;
	Composite composite;
	protected AssociationRole associationrole;
	Button sourceCheck;
	Button targetCheck;
	ClassifierRole source,target;
	Button ManhattanRoutingCheck;
	Button BendpointRoutingCheck;
	
	/**
	 * Constructs a new AssociationPropertiesDialog
	 * @param parentShell   parent shell (becomes blocked when this dialog is open)
	 * @param associationrole the AssociationRole to be edited
	 */
	public AssociationPropertiesDialog(Shell parentShell , AssociationRole associationrole)
	{
		super(parentShell);
		setShellStyle( getShellStyle() | SWT.RESIZE );
		
		this.associationrole = associationrole;
		this.source=associationrole.getAssociationSource();
		this.target=associationrole.getAssociationTarget();
		
		
		nameSourceClassifier  = associationrole.getAssociationSource().getName()  != null ? "\""+associationrole.getAssociationSource().getName()+": "+ associationrole.getAssociationSource().getType()+"\"" : ""; 
		nameTargetClassifier  = associationrole.getAssociationTarget().getName()!= null ? "\""+associationrole.getAssociationTarget().getName()+": "+associationrole.getAssociationTarget().getType()+"\""  : "";
		name =   associationrole.getName() != null ? associationrole.getName() : "";
	}
	
	
	/**  
	 * Constructs a new AssociationPropertiesDialog
	 * @param parentShell   parent shell (becomes blocked when this dialog is open)
	 * @param associationrole the AssociationRole to be edited
	 * @param source the instance of ClassifierRole connected with the associationrole as source
	 * @param target the instance of ClassifierRole conected with the associationrole as target
	 */
	public AssociationPropertiesDialog(Shell parentShell, AssociationRole associationrole, ClassifierRole source, ClassifierRole target )
	{
		super(parentShell);
		setShellStyle( getShellStyle() | SWT.RESIZE );
		
		this.associationrole = associationrole;
		this.source=source;
		this.target=target;
		
		nameSourceClassifier  = source.getName()  != null ? "\""+source.getName()+": "+ source.getType()+"\"" : ""; 
		nameTargetClassifier  = target.getName()!= null ? "\""+target.getName()+": "+ target.getType()+"\""  : "";
		name =   associationrole.getName() != null ? associationrole.getName() : "";
	}

	/**
	 * @author bjoern
	 * 
	 * Helper function, adding a Label with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Label addLabelToComposite( Composite composite, String text ) 
	{
		Label label = new Label( composite, 0 );
		label.setText( text );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 10; gd.grabExcessHorizontalSpace = true; 
		label.setLayoutData( gd );
		return label;
	}

	/**
	 * @author bjoern
	 * 
	 * Helper function, adding a Text (-field) with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Text addTextToComposite( Composite composite, String textValue, ModifyListener textListener ) 
	{
		Text text = new Text( composite, SWT.SINGLE | SWT.BORDER );
		text.setText( textValue );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		text.setLayoutData( gd );
		if ( textListener != null )
			text.addModifyListener( textListener );
		return text;
	}
	
	/**
	 * Helper function, adding a Button with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @return the button that was added to composite
	 */
	Button addButtonToComposite( Composite composite) 
	{
		Button button = new Button( composite, SWT.CHECK );
		button.addSelectionListener( this );
		GridData gd = new GridData(); 
		gd.widthHint = 90;
		button.setLayoutData( gd ); 
		return button;
	}
	
	/** 
	 * creates the content of the dialog
	 *  
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite superComposite = (Composite) super.createDialogArea(parent);
		composite = new Composite( superComposite, 0 );
		composite.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout( layout );
		
		addLabelToComposite( composite, "Navigation to classifier : ");
		addLabelToComposite( composite, " ");
		addLabelToComposite( composite, "\t"+nameSourceClassifier);
		sourceCheck=addButtonToComposite(composite);
		addLabelToComposite( composite, "\t"+nameTargetClassifier);
		targetCheck=addButtonToComposite(composite);
		targetCheck.setSelection(true);
		addLabelToComposite( composite, "Association Role end name: ");
		nameText  = addTextToComposite( composite, name, this );

		addLabelToComposite( composite, "Routing type: ");
		addLabelToComposite( composite, " ");
		addLabelToComposite( composite, "\tManhattan style routing ");
		ManhattanRoutingCheck  = addButtonToComposite( composite );
		ManhattanRoutingCheck.setSelection(associationrole.getRoutingType()==1);
		addLabelToComposite( composite, "\tBendpoint routing ");
		BendpointRoutingCheck  = addButtonToComposite( composite );
		BendpointRoutingCheck.setSelection(associationrole.getRoutingType()==0);

		return superComposite;
	}

	/**
	 * implementation of ModifyListener.modifyText() used to show error messages and
	 * disable ok button on incomplete input 
	 */
	public void modifyText(ModifyEvent e)
	{
		if ( nameText.getCharCount() == 0  ) {
			setErrorMessage( "You need to specify an association role end name!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		} else if ( nameText.getCharCount() >0 && existsWhiteSpace(nameText.getText())) {
			setErrorMessage( "Whitespace is not allowed in association name!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else {
			setErrorMessage( null );
			setMessage( "Edit the properties of your Association Role." );
			getButton(IDialogConstants.OK_ID).setEnabled( true );
		}
		name  = nameText.getText();
	}

	
	/**
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */	 
	protected Control createContents(Composite parent) {
		Control superContent = super.createContents(parent);
		
		parent.getShell().setMinimumSize( 300, 200 );
		parent.getShell().setText( "Association Role Properties" );
		nameText.setText( nameText.getText() );  // to call listeners
		
		return superContent;
	}
	
	private boolean existsWhiteSpace(String name){		
		String att = name;
		for(int j=0; j<att.length(); j++){
		 char c = att.charAt(j);
		 if(Character.isWhitespace(c))
			return true;
		}				
		return false;
	}
	
	/**
	 * Widged default selected listener.
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected( SelectionEvent e ) 
	{
		widgetSelected( e );
	}
	
	/**
	 * implementation of SelectionListener.widgetSelected() 
	 * toggle checkboxes 
	 */
	public void widgetSelected(SelectionEvent e) 
	{
		if ( e.getSource() == sourceCheck) {
			targetCheck.setSelection(false);
		} else if(e.getSource() == targetCheck) {
			sourceCheck.setSelection(false);
		} else if(e.getSource() == ManhattanRoutingCheck) {
			BendpointRoutingCheck.setSelection(false);
		} else if(e.getSource() == BendpointRoutingCheck) {
			ManhattanRoutingCheck.setSelection(false);
		}	
	}
	
	/**
	 * stores edited association role properties after user clicked the "ok"-button
	 * Swaps source & target and clears the bendpoint list if necessary
	 */
	protected void okPressed() {
		if (sourceCheck.getSelection()){
			source.getTarget().remove(associationrole);
			target.getSource().remove(associationrole);
			ClassifierRole source_helpvar=source;
			source=target;
			target=source_helpvar;
			
			// clear the Bendpoint lists
			associationrole.getBendpointsFirstHeightList().clear();
			associationrole.getBendpointsFirstWidthList().clear();
			associationrole.getBendpointsSecondHeightList().clear();
			associationrole.getBendpointsSecondWidthList().clear();
		}
		associationrole.setRoutingType((byte)(ManhattanRoutingCheck.getSelection()?1:0));
		super.okPressed();
	}

	/**
	 * Returns the name of the association end role
	 * @return the name of the association end role
	 */
	public String getName()
	{
		return name;
	}
	
	/**
	 * Returns the new instance of ClassifierRole connected with as source
	 * @return the new instance of ClassifierRole connected with as source
	 */	
	public ClassifierRole getSource() {
		return source;		
	}
	
	/**
	 * Returns the new instance of ClassifierRole connected with as target
	 * @return the new instance of ClassifierRole connected with as target
	 */
	public ClassifierRole getTarget() {
		return target;
	}
}
