/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.dialog;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;


import VOCLModel.*;

/**
 * @author manaljav
 * JFace based dialog used to edit condition properties 
 */
public class ConditionPropertiesDialog extends TitleAreaDialog implements SelectionListener, ModifyListener {
	Text  valueText;
	Combo opCombo , leftCombo ,rightCombo;
	String op, value, leftVar, rightVar;
	Condition condition;
	Composite composite;
	Composite superComposite;
	Constraint constraint;
	NavExp navExp;
	VariableDeclaration left,right;
	Button valueCheck;
	boolean valueBoolean = false;
	Button variableCheck;
	boolean variableBoolean = false;
	Button leftPreCheck;
	boolean leftPre = false;
	Button rightPreCheck;
	boolean rightPre = false;
	List leftComboList= new ArrayList();

	
	/**
	 * Constructs a new ConditionPropertiesDialog
	 * @param parentShell   parent shell (becomes blocked when this dialog is open)
	 * @param condition the {@link Condition} to be edited
	 */
	public ConditionPropertiesDialog(Shell parentShell, Condition condition, Constraint constraint, NavExp navExp ){
		super(parentShell);
		setShellStyle( getShellStyle() | SWT.RESIZE );
		this.condition = condition;
		this.constraint = constraint;
		this.navExp = navExp;
		
		op 		= condition.getOp()  	!= null ? condition.getOp()  	: ""; 
		value 	= condition.getValue() 	!= null ? condition.getValue() 	: "";
		left 	= condition.getLeft() 	!= null ? condition.getLeft()  	: null; 
		right 	= condition.getRight() 	!= null ? condition.getRight()  : null; 
		leftPre = condition.isLeftPre();
		rightPre = condition.isRightPre();
	}
	
	/**
	 * Helper function, adding a Button with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @return the button that was added to composite
	 */
	Button addButtonToComposite( Composite composite) 
	{
		Button button = new Button( composite, SWT.CHECK );
		button.addSelectionListener( this );
		GridData gd = new GridData(); 
		gd.widthHint = 20;
		button.setLayoutData( gd ); 
		return button;
	}

	/**
	 *  
	 * Helper function, adding a Label with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 * 
	 * @author bjoern
	 */
	Label addLabelToComposite( Composite composite, String text , int length) 
	{
		Label label = new Label( composite, 0 );
		label.setText( text );
		GridData gd = new GridData(); 
		gd.widthHint = length;
		label.setLayoutData( gd );
		return label;
	}

	/**
	 * Helper function, adding a Text (-field) with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Text addTextToComposite( Composite composite, String textValue, ModifyListener textListener ) 
	{
		Text text = new Text( composite, SWT.SINGLE | SWT.BORDER );
		text.setText( textValue );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		text.setLayoutData( gd );
		if ( textListener != null )
			text.addModifyListener( textListener );
		return text;
	}
	/**
	 *
	 * @param composite The composite to add the label to
	 * @param textValue The text of the label to be added to the composite
	 * @return the combo that was added to composite
	 */	
	Combo addComboToCompositeVar(Composite composite, String textValue) {
		
		Combo com = new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE | SWT.READ_ONLY);
    	com.setText( textValue );
    	GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		com.setLayoutData( gd );
		List variables = constraint.getAllVariables();
		Iterator it = variables.iterator();
		while(it.hasNext()){
			Object next = it.next();
			if(next instanceof VariableDeclaration) {
				VariableDeclaration var = (VariableDeclaration)next;
				String name = var.getName();
				
				if ( name != null && ! "".equals( name ) ){
					if(!existsVariable(com,var)){
						com.add(((VariableDeclaration)next).getName()+":"+((VariableDeclaration)next).getType());
					}
				}
			}
		}
		return com;
	}
	
	private boolean existsVariable(Combo combo,VariableDeclaration var){
		if(combo.indexOf(var.getName()+":"+var.getType())!=-1)
			return true;
		else 
			return false;
	}
	
	/**
	 *
	 * @param composite The composite to add the label to
	 * @param textValue The text of the label to be added to the composite
	 * @return the combo that was added to composite
	 */	
	Combo addComboToCompositeOp(Composite composite, String textValue) {
		
		Combo com = new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE | SWT.READ_ONLY);
		com.setText( textValue );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		com.setLayoutData( gd );
		com.add("<");
		com.add(">");
		com.add("<=");
		com.add(">=");
		com.add("=");
		com.add("!=");
		return com;
	}
			
	/** 
	 * Creates the content of the dialog.
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		superComposite = (Composite) super.createDialogArea(parent);
		composite = new Composite( superComposite, 0 );
		composite.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		composite.setLayout( layout );
		
		addLabelToComposite( composite, "Left Variable:", 130 );
		addLabelToComposite( composite, "" , 20);
		
		if (left == null)
			leftCombo  = addComboToCompositeVar( composite, "");
		else
			leftCombo  = addComboToCompositeVar( composite, left.getName());
		
	
		
		addLabelToComposite( composite, "Left Variable Pre:", 130 );
		addLabelToComposite( composite, "" , 20);
		leftPreCheck  = addButtonToComposite( composite);
		leftPreCheck.setSelection(leftPre);
		
		addLabelToComposite( composite, "Operation:", 130 );
		addLabelToComposite( composite, "" , 20);
		opCombo  = addComboToCompositeOp( composite, op);
		
		
		addLabelToComposite( composite, "Value:", 130 );
		valueCheck = addButtonToComposite(composite);
		valueText = addTextToComposite( composite, value, this ); 
		
		
		addLabelToComposite( composite, "Right Variable:", 130 );
		variableCheck = addButtonToComposite(composite);
		if (right == null)
			rightCombo  = addComboToCompositeVar( composite, "" );
		else
			rightCombo  = addComboToCompositeVar( composite, right.getName());
		
		
		addLabelToComposite( composite, "Right Variable Pre:", 130 );
		addLabelToComposite( composite, "" , 20);
		rightPreCheck  = addButtonToComposite( composite);
		rightPreCheck.setSelection(rightPre);
		
		if(op!=null){
			opCombo.select(this.finder(opCombo,op));
		}
		else
			opCombo.select(0);
		if(left!=null){
			leftCombo.select(this.finder(leftCombo,left.getName()+":"+left.getType())); 
			leftPreCheck.setSelection(leftPre);
		}
			
		if(right != null){
			variableCheck.setSelection(true);
			variableBoolean = true;
			valueText.setEditable(false);
			rightCombo.select(this.finder(rightCombo,right.getName()+":"+right.getType())); 
			rightPreCheck.setSelection(rightPre);
		} else {
			valueCheck.setSelection(true);
			valueBoolean = true;
			rightCombo.setEnabled(false);
			rightPreCheck.setEnabled(false);
		}
	
		opCombo.addSelectionListener( this );
		leftCombo.addSelectionListener( this );
		rightCombo.addSelectionListener( this );
		leftPreCheck.addSelectionListener(this);
		rightPreCheck.addSelectionListener( this );
		
		return superComposite;
	}

	/**
	 * implementation of ModifyListener.modifyText() used to show error messages and
	 * disable ok button on incomplete input 
	 */
	public void modifyText(ModifyEvent e)
	{
		
		if ( leftCombo.getText() == "" ) {
			setErrorMessage( "You need to specify a left value!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else if ( opCombo.getText() == "" ) {
			setErrorMessage( "You need to specify the operation!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		} else if(valueCheck.getSelection() && valueText.getCharCount() == 0){
			setErrorMessage( "You need to specify a value!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		} else if( (!valueCheck.getSelection())&&
					!(((	(leftCombo.getText()).substring((leftCombo.getText()).indexOf(":")+1,leftCombo.getText().length()))
				      .equals((rightCombo.getText()).substring((rightCombo.getText()).indexOf(":")+1,rightCombo.getText().length()))))
			){
			setErrorMessage( " Variables must have same type" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		} else if(variableCheck.getSelection() && (rightCombo.getText()=="" )){
			setErrorMessage( "You need to specify a right variable name!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		} else if(((leftCombo.getText()).equals(rightCombo.getText()))&&(!(opCombo.getText().equals("=")))&&(leftPre==rightPre)){
			setErrorMessage( " Please choose another variable or operation!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		} else {
			setErrorMessage( null );
			setMessage( "Edit the properties of your Condition." );
			getButton(IDialogConstants.OK_ID).setEnabled( true );
		}
		op  = opCombo.getText();
		value = valueText.getText();
		String leftVariableName = "";
		if(leftCombo.getText().indexOf(":") != -1){
			leftVariableName = (leftCombo.getText()).substring(0,(leftCombo.getText()).indexOf(":"));
		}else{
			leftVariableName = leftCombo.getText();
		}
		for(int i=0; i<constraint.getAllVariables().size(); i++){
			if(leftVariableName.equals(((VariableDeclaration) constraint.getAllVariables().get(i)).getName())){
				left = (VariableDeclaration) constraint.getAllVariables().get(i);
				break;
			}			
		}
		String rightVariableName = "";
		if(rightCombo.getText().indexOf(":") != -1){
			rightVariableName = (rightCombo.getText()).substring(0,(rightCombo.getText()).indexOf(":"));
		}else{
			rightVariableName = rightCombo.getText();
		}
		for(int i=0; i<constraint.getAllVariables().size(); i++){
			if(rightVariableName.equals(((VariableDeclaration) constraint.getAllVariables().get(i)).getName())){
				right = (VariableDeclaration) constraint.getAllVariables().get(i);
				break;
			}			
		}
		leftPre = leftPreCheck.getSelection();
		rightPre = rightPreCheck.getSelection();
	}
	
	/**
	 * Creates contents.
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		Control superContent = super.createContents(parent);
		
		parent.getShell().setMinimumSize( 400, 300 );
		parent.getShell().setText( "Condition Properties" );
				
		modifyText(null);
		return superContent;
	}
	
	/**
	 * Cancel pressed.
	 * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
	 */
	protected void cancelPressed() {
		super.cancelPressed();
	}
	
	/**
	 * sets the edited attribute properties after user clicks "OK"
	 */
	protected void okPressed() {
		super.okPressed();
	}
	
	/**
	 * Returns the operation.
	 * @return the operation
	 */
	public String getOp() {
		return op;
	}
	
	/**
	 * Returns the value.
	 * @return the value
	 */
	public String getValue() {
		if(! valueBoolean)
			return null;
		return value;
	}
	
	/**
	 * Returns the left variable.
	 * @return the left variable
	 */
	public VariableDeclaration getLeft(){
		return left;
	}
	
	/**
	 * Returns true if the left variable is pre.
	 * @return true if the left variable is pre
	 */
	public boolean getLeftPre(){
		return leftPre;
	}
	
	/**
	 * Return true if the right variable is pre.
	 * @return true if the right variable is pre
	 */
	public boolean getRightPre(){
		return rightPre;
	}
	
	/**
	 * Returns the right variable.
	 * @return the right variable
	 */
	public VariableDeclaration getRight(){
		if(! variableBoolean)
			return null;		
		return right;
	}
	
	/**
	 * Widged selected listener.
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent event) {		
		if ( event.getSource() == valueCheck) {
			variableCheck.setSelection(false);
			valueCheck.setSelection(true);
			valueBoolean = true;
			variableBoolean = false;
			rightCombo.setEnabled(false);
			rightPreCheck.setEnabled(false);
			valueText.setEditable(true);
			
		} else if(event.getSource() == variableCheck) {
			variableCheck.setSelection(true);
			valueCheck.setSelection(false);
			variableBoolean = true;
			valueBoolean = false;		
			valueText.setEditable(false);
			rightCombo.setEnabled(true);
			rightPreCheck.setEnabled(true);
		}
		modifyText(null);
	}
	
	/**
	 * Widged default selected listener.
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected(SelectionEvent event) {
		widgetSelected(event);
	}
	
	private int finder(Combo combo, String str){
	   	String[] items = combo.getItems();
		int i = 0; 
		while(i < items.length){
			if(items[i].equals(str))
				break;		
			i++;
		}
		return i;
	}	
	
}
