/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.dialog;

import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Slider;

import VOCLModel.IfExp;

/**
 * Dialog to modify Bool Exp properties
 * @author bjoern
 */
public class IfExpPropertiesDialog 
	extends TitleAreaDialog
	implements SelectionListener, ModifyListener
{
	IfExp    			 ifexp;
	boolean    			 isHorizontal, wasHorizontal;
	boolean				 negated;
	int					 splitPercent1, splitPercent2;
	int					 oldSplitPercent1, oldSplitPercent2;
	
	Button     horizontalButton, negateButton; 
	Slider     splitSlider1, splitSlider2;
	Label		splitLabel1, splitLabel2;
	
	/**
	 * Constructs a IfExpPropertiesDialog
	 * @param parentShell parent shell
	 * @param ifexp the IfExp to be shown/modified
	 */
	public IfExpPropertiesDialog(Shell parentShell, IfExp ifexp )
	{   
		super(parentShell);
		this.ifexp = ifexp;
		isHorizontal = ifexp.isSplitHorizontal();
		wasHorizontal = isHorizontal;
		splitPercent1 = ifexp.getSplitPercent1();
		splitPercent2 = ifexp.getSplitPercent2();
		oldSplitPercent1 = splitPercent1;
		oldSplitPercent2 = splitPercent2;
		negated = ifexp.isNegated();
		setShellStyle( getShellStyle() | SWT.RESIZE );
	}
		
	/**
	 * 
	 * Helper function, adding a Label with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Label addLabelToComposite( Composite composite, String text ) 
	{
		Label label = new Label( composite, 0 );
		label.setText( text );
		GridData gd = new GridData(); gd.widthHint = 180;
		label.setLayoutData( gd );
		return label;
	}

	/**
	 * 
	 * Helper function, adding a Slider with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Slider addSliderToComposite( Composite composite, int value, int maximum, int minimum ) 
	{
		Slider slider = new Slider( composite, SWT.HORIZONTAL );
		slider.setValues( value, minimum, maximum, 10, 1, 10 );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); 
		gd.widthHint = 180;
		slider.setLayoutData( gd );
		slider.addSelectionListener( this );
		return slider;
	}

	
	
	/**
	 * 
	 * Helper function, adding a Button with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the button label
	 * @return the button that was added to composite
	 */
	Button addButtonToComposite( Composite composite, String text ) 
	{
		Button button = new Button( composite, SWT.CHECK );
		button.setText( text );
		button.addSelectionListener( this );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); 
		gd.widthHint = 90;
		button.setLayoutData( gd ); 
		return button;
	}
	
	/**
	 * 
	 * Helper function, adding a ComboBox with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the button label
	 * @return the button that was added to composite
	 */	
    Combo addComboToComposite( Composite composite, String textValue ) {
    	Combo com = new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE | SWT.READ_ONLY);
    	com.setText( textValue );
    	GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
    	com.setLayoutData( gd );
    	com.addModifyListener( this );
		return com;
    }

    /** 
	 * Creates the content of the dialog.
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{ 
		Composite superComposite = (Composite) super.createDialogArea(parent);
		
		Composite composite = new Composite( superComposite , 0 );
		composite.setLayoutData( new GridData( GridData.FILL_BOTH ) );

		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout( layout );
		
		addLabelToComposite( composite, "Horizontal:" );
		horizontalButton = addButtonToComposite( composite, "split expressions horizontally");
		horizontalButton.setSelection( isHorizontal );
		setMessage( "Edit the properties of your If-Then-Else expression." );

		splitLabel1 = addLabelToComposite( composite, "Split If <-> Then at: " + splitPercent1 + "%");
		splitSlider1 = addSliderToComposite( composite, splitPercent1, 90, 20 );

		splitLabel2 = addLabelToComposite( composite, "Split Then <-> Else at: " + splitPercent2 + "%");
		splitSlider2 = addSliderToComposite( composite, splitPercent2, 90, 20 );

		addLabelToComposite( composite, "Negated:" );
		negateButton = addButtonToComposite( composite, "negate the whole expressions");
		negateButton.setSelection( negated );
		
		return composite;
	}

	/**
	 * Create contents.
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */	 
	protected Control createContents(Composite parent) 
	{	
		Control superContent = super.createContents(parent);
		
		parent.getShell().setSize( 550, 300 );
		parent.getShell().setMinimumSize( 500, 300 );
		parent.getShell().setText( "If-Then-Else Expression's Properties" );
		return superContent;
	}
	
	/**
	 * Cancel pressed.
	 * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
	 */
	protected void cancelPressed() {
		ifexp.setSplitPercent1( oldSplitPercent1 );
		ifexp.setSplitPercent2( oldSplitPercent2 );
		ifexp.setSplitHorizontal( wasHorizontal );
		isHorizontal = wasHorizontal;
		splitPercent1 = oldSplitPercent1;
		splitPercent2 = oldSplitPercent2;
		super.cancelPressed();
	}
	
	/**
	 * Returns the context type selected in this dialog ("inv", "pre" or "post").
	 * @return the context type selected in this dialog ("inv", "pre" or "post")
	 */
	public boolean getIsHorizontal() {
		return isHorizontal;
	}

	/**
	 * Returns the split percentage 1.
	 * @return the split percentage 1
	 */
	public int getSplitPercentage1() {
		return splitPercent1;
	}

	/**
	 * Returns the split percentage 2.
	 * @return the split percentage 2
	 */
	public int getSplitPercentage2() {
		return splitPercent2;
	}

	/**
	 * Returns the negation flag.
	 * @return the negation flag
	 */
	public boolean isNegated() {
		return negated;
	}
	
	/**
	 * listens and reacts to "Add", "Edit" and "Delete" buttons
	 */
	public void widgetSelected(SelectionEvent e) 
	{
		if ( e.getSource() == horizontalButton ) {
			isHorizontal = horizontalButton.getSelection();
			ifexp.setSplitHorizontal( isHorizontal ); // "the preview effect" (tm)
		} else if ( e.getSource() == negateButton ) {
			negated = negateButton.getSelection();
		} else if ( e.getSource() == splitSlider1 ) {
			splitPercent1 = splitSlider1.getSelection();
			splitLabel1.setText( "Split If <-> Then at: " + splitPercent1 + "%" );
			ifexp.setSplitPercent1( splitPercent1 );  // "the preview effect" (tm)
		} else if ( e.getSource() == splitSlider2 ) {
			splitPercent2 = splitSlider2.getSelection();
			splitLabel2.setText( "Split Then <-> Else at: " + splitPercent2 + "%" );
			ifexp.setSplitPercent2( splitPercent2 );  // "the preview effect" (tm)
		}
	}
	
	/**
	 * Widged default selected listener.
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected( SelectionEvent e ) 
	{
		widgetSelected( e );
	}
	
	/**
	 * Modifies text.
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		//operation = opCombo.getText();		
	}
}
