/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.dialog;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Slider;

import vocleditor.model.VOCLTools;

import VOCLModel.Attribute;
import VOCLModel.ClassifierRole;
import VOCLModel.LetExp;
import VOCLModel.NavExp;
import VOCLModel.VariableDeclaration;
import VOCLModel.SetClassifierRole;

/**
 * Dialog to modify Bool Exp properties
 * @author bjoern
 * TODO jdoc
 */
/**
 * @author tugstugi
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class LetExpPropertiesDialog 
	extends TitleAreaDialog
	implements SelectionListener, ModifyListener
{
	LetExp  	letexp;
	boolean    	isHorizontal, wasHorizontal;
	int			splitPercent;
	int			oldSplitPercent;
	boolean		negated;
	
	Button		horizontalButton, negateButton; 
	Slider		splitSlider;
	Label		splitLabel;
	
	Combo		varCombo;
	
	Vector		classifierVars = new Vector();
	Vector		attributeVars = new Vector();
	Vector		methodVars = new Vector();
	Vector		setVars = new Vector();
	int			classifierIdx, methodIdx, attrIdx, setIdx;
	VariableDeclaration varDecl;
	
	/**
	 * Constructs a LetExpPropertiesDialog
	 * @param parentShell parent shell
	 * @param letexp the LetExp to be shown/modified
	 */
	public LetExpPropertiesDialog(Shell parentShell, LetExp letexp )
	{   
		super(parentShell);
		this.letexp = letexp;
		this.varDecl = letexp.getLetVar();
		isHorizontal = letexp.isSplitHorizontal();
		wasHorizontal = isHorizontal;
		splitPercent = letexp.getSplitPercent();
		oldSplitPercent = splitPercent;
		negated = letexp.isNegated();
		setShellStyle( getShellStyle() | SWT.RESIZE );
	}
		
	/**
	 * 
	 * Helper function, adding a Label with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Label addLabelToComposite( Composite composite, String text ) 
	{
		Label label = new Label( composite, 0 );
		label.setText( text );
		GridData gd = new GridData(); gd.widthHint = 150;
		label.setLayoutData( gd );
		return label;
	}

	/**
	 * 
	 * Helper function, adding a Slider with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Slider addSliderToComposite( Composite composite, int value, int maximum, int minimum ) 
	{
		Slider slider = new Slider( composite, SWT.HORIZONTAL );
		slider.setValues( value, minimum, maximum, 10, 1, 10 );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); 
		gd.widthHint = 150;
		slider.setLayoutData( gd );
		slider.addSelectionListener( this );
		return slider;
	}

	
	
	/**
	 * 
	 * Helper function, adding a Button with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the button label
	 * @return the button that was added to composite
	 */
	Button addButtonToComposite( Composite composite, String text ) 
	{
		Button button = new Button( composite, SWT.CHECK );
		button.setText( text );
		button.addSelectionListener( this );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); 
		gd.widthHint = 90;
		button.setLayoutData( gd ); 
		return button;
	}
	
	/**
	 * 
	 * Helper function, adding a ComboBox with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the button label
	 * @return the button that was added to composite
	 */	
    Combo addComboToComposite( Composite composite, String textValue ) {
    	Combo com = new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE | SWT.READ_ONLY);
    	com.setText( textValue );
    	GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
    	com.setLayoutData( gd );
    	com.addModifyListener( this );
		return com;
    }

    /** 
	 * Creates the content of the dialog.
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{ 
		Composite superComposite = (Composite) super.createDialogArea(parent);
		
		Composite composite = new Composite( superComposite , 0 );
		composite.setLayoutData( new GridData( GridData.FILL_BOTH ) );

		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout( layout );
		
		addLabelToComposite( composite, "Variable: " );
		varCombo = addComboToComposite( composite, "choose" ); 
		fillVariableCombo();
		varCombo.setText( "Los gehts!" );
		
		addLabelToComposite( composite, "Horizontal:" );
		horizontalButton = addButtonToComposite( composite, "split expressions horizontally");
		horizontalButton.setSelection( isHorizontal );

		splitLabel = addLabelToComposite( composite, "Split at: " + splitPercent + "%");
		splitSlider = addSliderToComposite( composite, splitPercent, 90, 20 );

		addLabelToComposite( composite, "Negated:" );
		negateButton = addButtonToComposite( composite, "negate the whole expressions");
		negateButton.setSelection( negated );

		
		setMessage( "Edit the properties of your Let expression." );
		return composite;
	}

	/**
	 * Creates contents.
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */	 
	protected Control createContents(Composite parent) 
	{	
		Control superContent = super.createContents(parent);
		
		parent.getShell().setSize( 550, 300 );
		parent.getShell().setMinimumSize( 500, 300 );
		parent.getShell().setText( "Let Expression's Properties" );
		return superContent;
	}
	
	/**
	 * Cancel pressed.
	 * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
	 */
	protected void cancelPressed() {
		letexp.setSplitPercent( oldSplitPercent );
		letexp.setSplitHorizontal( wasHorizontal );
		isHorizontal = wasHorizontal;
		splitPercent = oldSplitPercent;
		super.cancelPressed();
	}
	
	/**
	 * Returns the context type selected in this dialog ("inv", "pre" or "post").
	 * @return the context type selected in this dialog ("inv", "pre" or "post")
	 */
	public boolean getIsHorizontal() {
		return isHorizontal;
	}

	
	/**
	 * Returns the split percentage.
	 * @return the split percantage
	 */
	public int getSplitPercentage() {
		return splitPercent;
	}
		
	/**
	 * Returns the variable declaration.
	 * @return the variable declaration
	 */
	public VariableDeclaration getVariableDeclaration()
	{
		return varDecl;
	}
	
	/**
	 * Returns the negation flag.
	 * @return the negation flag
	 */
	public boolean isNegated() {
		return negated;
	}
	
	/**
	 * listens and reacts to "Add", "Edit" and "Delete" buttons
	 */
	public void widgetSelected(SelectionEvent e) 
	{
		if ( e.getSource() == horizontalButton ) {
			isHorizontal = horizontalButton.getSelection();
			letexp.setSplitHorizontal( isHorizontal ); // "the preview effect" (tm)
		} else if ( e.getSource() == negateButton ) {
			negated = negateButton.getSelection();
		} else if ( e.getSource() == splitSlider ) {
			splitPercent = splitSlider.getSelection();
			splitLabel.setText( "Split at: " + splitPercent + "%" );
			letexp.setSplitPercent( splitPercent );  // "the preview effect" (tm)
		}
	}
	
	/**
	 * Widged default selected listener.
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected( SelectionEvent e ) 
	{
		widgetSelected( e );
	}

	/**
	 * Modifies text.
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		int si = varCombo.getSelectionIndex();
		if ( si == classifierIdx || si == methodIdx || si == attrIdx || si == setIdx) {
			si++;
			varCombo.select( si );
		}
		if ( si > attrIdx )
			varDecl = (VariableDeclaration) attributeVars.get( (si - attrIdx) - 1 );
		else if ( si > methodIdx )
			varDecl = (VariableDeclaration) methodVars.get( (si - methodIdx) - 1 );
		else if ( si > setIdx )
			varDecl = (VariableDeclaration) setVars.get( (si - setIdx) - 1 );	
		else if ( si > classifierIdx )
			varDecl = (VariableDeclaration) classifierVars.get( (si - classifierIdx) - 1 );	
	}
	
	private void addVarToList( List list, VariableDeclaration var )
	{
		boolean alreadyThere = false;
		for ( Iterator i = list.iterator(); i.hasNext(); ) {
			VariableDeclaration v = (VariableDeclaration) i.next();
			if ( var.getName().equals( v.getName() ) && var.getType().equals( v.getType() )) {
				alreadyThere = true;
				break;
			}
		}
		if ( ! alreadyThere )
			list.add( var );
	}

	private void fillVariableCombo()
	{
		//List classifiers = ((NavExp) letexp.getLet()).getClassifierRoles();
		List classifiers = VOCLTools.getAllClassifiers( (NavExp) letexp.getLet(), new Vector() );
		for ( int i = 0; i < classifiers.size(); i++ ) {
			ClassifierRole c = (ClassifierRole)classifiers.get(i);
			if ( c.getVariable() != null )
				addVarToList( classifierVars, c.getVariable() );
			if ( c.getMethod() != null && c.getMethod().getReturnVariable() != null )
				addVarToList( methodVars, c.getMethod().getReturnVariable() );
			List attrs = c.getAttributes();
			
			if(c instanceof SetClassifierRole) {
				SetClassifierRole s = (SetClassifierRole)c;
				if(s.getSum() != null)
					addVarToList( setVars, s.getSum());
				if(s.getSize() != null)
					addVarToList( setVars, s.getSize());
			}
			
			for ( int j = 0; j < attrs.size(); j++ ) {
				Attribute attr = (Attribute) attrs.get(j);
				if ( attr.getVariable() != null )
					addVarToList( attributeVars, attr.getVariable() );
			}				
		}
		
		classifierIdx = methodIdx = attrIdx = setIdx = 0x7FFFFFFF;

		if ( classifierVars.size() > 0 ) {
			classifierIdx = 0;
			varCombo.add( "Classifier variables:" );
			for ( int i = 0; i < classifierVars.size(); i++ ) {
				VariableDeclaration var = (VariableDeclaration) classifierVars.get(i);			
				varCombo.add( "    " + var.getName() + " : " + var.getType() );
				if ( varDecl == var ) varCombo.select( varCombo.getItemCount() - 1 );			
			}
		}

		if ( setVars.size() > 0 ) {
			setIdx = varCombo.getItemCount();;
			varCombo.add( "SetClassifier variables:" );
			for ( int i = 0; i < setVars.size(); i++ ) {
				VariableDeclaration var = (VariableDeclaration) setVars.get(i);			
				varCombo.add( "    " + var.getName() + " : " + var.getType() );
				if ( varDecl == var ) varCombo.select( varCombo.getItemCount() - 1 );			
			}
		}

		if ( methodVars.size() > 0 ) {
			methodIdx = varCombo.getItemCount();
			varCombo.add( "Method return variables:" );
			for ( int i = 0; i < methodVars.size(); i++ ) {
				VariableDeclaration var = (VariableDeclaration) methodVars.get(i);
				varCombo.add( "    " + var.getName() + " : " + var.getType() );
				if ( varDecl == var ) varCombo.select( varCombo.getItemCount() - 1 );			
			}
		}

		if ( attributeVars.size() > 0 ) {
			attrIdx = varCombo.getItemCount();
			varCombo.add( "Attribute variables:" );
			for ( int i = 0; i < attributeVars.size(); i++ ) {
				VariableDeclaration var = (VariableDeclaration) attributeVars.get(i);
				varCombo.add( "    " + var.getName() + " : " + var.getType() );
				if ( varDecl == var ) varCombo.select( varCombo.getItemCount() - 1 );			
			}
		}
	}
	
}
