/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.dialog;

import java.util.Hashtable;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

import vocleditor.VOCLEditorPlugin;
import vocleditor.model.ModelCreationFactory;

import VOCLModel.ClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.Method;
import VOCLModel.VariableDeclaration;

/**
 * @author tugstugi
 * Dialog to modify method properties
 */
public class MethodPropertiesDialog extends TitleAreaDialog implements SelectionListener, ModifyListener {
	private Text methodNameText;
	private Text returnVariableNameText;
	private Combo returnTypeCombo;
	private String methodName;
	private String returnVariableName;
	private String returnType;
	private Constraint constraint;
	private ClassifierRole classifierRole;
	private Method method;
	private Table table;	
	private Button addParameterButton,chooseParameterButton, editParameterButton, deleteParameterButton, upParameterButton, downParameterButton;
	private List parameterList = new ArrayList();
	private Hashtable changedParameters = new Hashtable();

	/**
	 * Constructs a ClassifierPropertiesDialog
	 * @param parentShell the parent shell
	 * @param constraint the constraint
	 * @param classifierRole the classifier role
	 * @param method the Method to be shown/modified
	 */
	public MethodPropertiesDialog(Shell parentShell, Constraint constraint, ClassifierRole classifierRole, Method method) {   
		super(parentShell);
		this.constraint = constraint;
		this.classifierRole = classifierRole;
		this.method = method;
		parameterList = getInitialParameters(method);
		setShellStyle( getShellStyle() | SWT.RESIZE );
		methodName = method.getName() != null ? method.getName() : ""; 
		returnVariableName = method.getReturnVariable() != null ? method.getReturnVariable().getName() : "";
		returnType = method.getReturnType() != null ? method.getReturnType() : "";
	}

		
	/**
	 * 
	 * Helper function, adding a Label with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Label addLabelToComposite( Composite composite, String text ){
		Label label = new Label( composite, 0 );
		label.setText( text );
		GridData gd = new GridData(); gd.widthHint = 100;
		label.setLayoutData( gd );
		return label;
	}

	/**
	 * 
	 * Helper function, adding a Text (-field) with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param textValue The text of the Text
	 * @return the text that was added to composite
	 */
	Text addTextToComposite( Composite composite, String textValue ) {
		Text text = new Text( composite, SWT.SINGLE | SWT.BORDER );
		text.setText( textValue );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		text.setLayoutData( gd );
		text.addModifyListener( this );
		return text;
	}

	/**
	 * 
	 * Helper function, adding a Button with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the button label
	 * @return the button that was added to composite
	 */
	Button addButtonToComposite( Composite composite, String text ) {
		Button button = new Button( composite, SWT.PUSH );
		button.setText( text );
		button.addSelectionListener( this );
		GridData gd = new GridData(); 
		gd.widthHint = 90;
		button.setLayoutData( gd ); 
		return button;
	}
	
	/**
	 * Helper function, adding a Combo with GridLayout-Data to a Composite
	 * @param composite The composite to add the label to
	 * @param textValue The text of the Text
	 * @return the combobox that was added to composite
	 */
    Combo addComboToComposite(Composite composite, String textValue) {
    	Combo com=new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE);
    	com.setText(textValue);
    	GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		com.setLayoutData( gd );
		com.addModifyListener( this );
		return com;
    }

	/**
	 * Implementation of ModifyListener.modifyText() used to show error messages and
	 * disable ok button on incomplete input 
	 */
	public void modifyText(ModifyEvent e) {		
		if ( methodNameText.getText().length() == 0 ) {
			setErrorMessage("You must at least specify the name of your Method!");
			getButton(IDialogConstants.OK_ID).setEnabled(false);
		}
		/*else if(returnVariableNameText.getText().length() == 0 && returnTypeCombo.getText().length() != 0){
			setErrorMessage("You must at least specify the name of the return variable if you want a return value!");
			getButton(IDialogConstants.OK_ID).setEnabled(false);
		}
		*/
		else if ( methodNameText.getCharCount() >0 && existsWhiteSpace(methodNameText.getText())) {
			setErrorMessage( "Whitespace is not allowed in method name!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else if ( returnVariableNameText.getCharCount() >0 && existsWhiteSpace(returnVariableNameText.getText())) {
			setErrorMessage( "Whitespace is not allowed in return variable name!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else if(returnVariableNameText.getText().length() != 0 && returnTypeCombo.getText().length() == 0){
			setErrorMessage("You must at least specify return type of the method if you want a return value!");
			getButton(IDialogConstants.OK_ID).setEnabled(false);
		}
		else {
			setErrorMessage( null );
			setMessage("Edit the properties of your Method.");
			getButton(IDialogConstants.OK_ID).setEnabled(true);
		}
		methodName = methodNameText.getText();
		returnVariableName = returnVariableNameText.getText();
		returnType = returnTypeCombo.getText();
	}	
	
	/** 
	 * Creates the content of the dialog
	 */
	protected Control createDialogArea(Composite parent) {  
		Composite superComposite = (Composite) super.createDialogArea(parent);		
		Composite composite = new Composite( superComposite , 0 );
		composite.setLayoutData( new GridData( GridData.FILL_BOTH ) );
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout( layout );		
		addLabelToComposite( composite, "Method Name:" );
		methodNameText = addTextToComposite( composite, methodName );
		addLabelToComposite( composite, "Return Varname:" );
		returnVariableNameText = addTextToComposite( composite, returnVariableName ); 
		addLabelToComposite( composite, "Return Type:" );
		returnTypeCombo = addComboToComposite( composite, returnType );	 
		List types = VOCLEditorPlugin.getAllTypes(constraint);
		for(int i=0; i<types.size(); i++){
			returnTypeCombo.add((String) types.get(i));
		}
		Group attributeWidgets = new Group( composite, 0 );
		attributeWidgets.setText( "Parameters" );		
		attributeWidgets.setLayoutData( new GridData( SWT.FILL, SWT.FILL, true, true, 2, 1 ) );
		layout = new GridLayout();
		layout.numColumns = 2;
		attributeWidgets.setLayout( layout );
		table = new Table( attributeWidgets, SWT.BORDER );
		table.setHeaderVisible( true );
		table.setLayoutData( new GridData( SWT.FILL, SWT.FILL, true, true, 1, 6 ) );		
		TableColumn column = new TableColumn( table, 0 );
		column.setText("Name");
		column.setWidth(205);
		column = new TableColumn( table, 0 );
		column.setText("Type");
		column.setWidth(250);
		chooseParameterButton = addButtonToComposite( attributeWidgets, "Choose" ); 
		addParameterButton = addButtonToComposite( attributeWidgets, "Add" );
		editParameterButton = addButtonToComposite( attributeWidgets, "Edit" );
		downParameterButton = addButtonToComposite( attributeWidgets, "Down" );
		upParameterButton = addButtonToComposite( attributeWidgets, "Up" );
		deleteParameterButton = addButtonToComposite( attributeWidgets, "Delete" );
		fillParameterVariableTable();		
		return composite;
	}

	/**
	 * Creates contents.
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {		
		Control superContent = super.createContents(parent);		
		parent.getShell().setSize( 550, 550 );
		parent.getShell().setMinimumSize( 400, 400 );
		parent.getShell().setText( "Method Properties" );
		methodNameText.setText( methodNameText.getText() );  // to call listeners		
		return superContent;
	}
	
	private boolean existsWhiteSpace(String name){		
		String att = name;
		for(int j=0; j<att.length(); j++){
		 char c = att.charAt(j);
		 if(Character.isWhitespace(c))
			return true;
		}		
		return false;
	}
	
	/**
	 * Cancel pressed.
	 * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
	 */
	protected void cancelPressed() {
		super.cancelPressed();
	}
	
	/**
	 * Ok pressed.
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		super.okPressed();
	}
	
	/**
	 * Returns the method name.
	 * @return the method name
	 */
	public String getName() {
		return methodName;
	}
	
	/**
	 * Returns the method return type.
	 * @return the method return type
	 */
	public String getReturnType() {
		return returnType;
	}
	
	/**
	 * Returns the list of parameters.
	 * @return the list of parameters
	 */
	public List getParameters() {
		return parameterList;
	}
	
	/**
	 * Returns the hashtable of changed parameters.
	 * @return the hashtable of changed parameters
	 */
	public Hashtable getChangedParameters(){
		return changedParameters;
	}
	
	/**
	 * Returns the return variable name.
	 * @return the return variable name
	 */
	public String getReturnVariableName(){
		if("".equals(returnVariableName))
			return null;
		return returnVariableName;
	}
	
	private List getNewVariableTypes(){
		List newTypes = new Vector();
		for(int i=0; i<parameterList.size(); i++){
			newTypes.add(((VariableDeclaration) parameterList.get(i)).getType());
		}
		return newTypes;
	}
	
	private void refreshReturnTypeCombo(){
		List newTypes = getNewVariableTypes();
		for(int i=0; i<newTypes.size(); i++){
			if(returnTypeCombo.indexOf((String) newTypes.get(i)) == -1)
				returnTypeCombo.add((String) newTypes.get(i));
		}
	}
	
	/**
	 * listens and reacts to "Add", "Edit" and "Delete" buttons
	 */
	public void widgetSelected(SelectionEvent e) {
		if ( e.getSource() == addParameterButton )  {
			addMethodParameterVariable();
		    fillParameterVariableTable();
		    table.setSelection(parameterList.size() - 1);
		    refreshReturnTypeCombo();
		}
		else if ( e.getSource() == chooseParameterButton )  {
			chooseMethodParameterVariable();
		    fillParameterVariableTable();
		    table.setSelection(parameterList.size() - 1);
		  
		}else if ( e.getSource() == editParameterButton ) { 
			TableItem tis[] = table.getSelection();
			if ( tis.length > 0 ) {
				VariableDeclaration oldVariable = (VariableDeclaration) tis[0].getData();
				VariableDeclarationPropertiesDialog apd = new VariableDeclarationPropertiesDialog( getShell(), constraint, classifierRole, method, oldVariable, getNewVariableTypes());
				apd.open();
				if ( apd.getReturnCode() == Dialog.OK ) {
					int i = parameterList.indexOf(oldVariable);
					VariableDeclaration variable = null;
					if(getInitialParameters(method).contains(oldVariable)){
						variable = createMethodParameterVariable();
						changedParameters.put(variable, oldVariable);
					}else
						variable = oldVariable;
					variable.setName(apd.getName());
					variable.setType(apd.getType());
					parameterList.set(i, variable);
					fillParameterVariableTable();
					table.setSelection(i);
				}
			}
			refreshReturnTypeCombo();
		}
		else if ( e.getSource() == deleteParameterButton ) { 
			TableItem tis[] = table.getSelection();
			if ( tis.length > 0 ) {
				for ( int i = 0; i < tis.length; i++ ) { 
					VariableDeclaration attr = (VariableDeclaration) tis[0].getData();
					parameterList.remove(attr);
					if(changedParameters.containsKey(attr))
						changedParameters.remove(attr);
		            fillParameterVariableTable();		            
				}
			}
			table.setSelection(parameterList.size() - 1);
		}		
		else if(e.getSource() == upParameterButton){
			int index = table.getSelectionIndex();
			if(index > 0){
				Object selectedObject = parameterList.get(index);
				parameterList.set(index, parameterList.get(index - 1));
				parameterList.set(index - 1, selectedObject);
				fillParameterVariableTable();
				table.setSelection(index - 1);
			}
		}
		else if(e.getSource() == downParameterButton){
			int index = table.getSelectionIndex();
			if(index < parameterList.size() - 1){
				Object selectedObject = parameterList.get(index);
				parameterList.set(index, parameterList.get(index + 1));
				parameterList.set(index + 1, selectedObject);
				fillParameterVariableTable();
				table.setSelection(index + 1);
			}
		}
	}

	/**
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected( SelectionEvent e ){
		widgetSelected( e );
	}
	
	private VariableDeclaration createMethodParameterVariable() {
		VariableDeclaration variable = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
		return variable;		
	}
	
	// called after "Add" (add parameter to method) button 
	private void addMethodParameterVariable(){
		VariableDeclaration methodVar = createMethodParameterVariable();
		VariableDeclarationPropertiesDialog apd = new VariableDeclarationPropertiesDialog( getShell(), constraint, classifierRole, method, methodVar, getNewVariableTypes());
		apd.open();
		if ( apd.getReturnCode() == Dialog.OK ) {
			methodVar.setName(apd.getName());
			methodVar.setType(apd.getType());
			parameterList.add(methodVar);
		}
	}
	private void chooseMethodParameterVariable(){
		VariableDeclaration methodVar = createMethodParameterVariable();
		VariableChoosePropertiesDialog apd = new VariableChoosePropertiesDialog( getShell(), constraint, classifierRole, method, methodVar);
		apd.open();
		if ( apd.getReturnCode() == Dialog.OK ) {
			methodVar.setName(apd.getName());
			methodVar.setType(apd.getType());
			parameterList.add(methodVar);
		}
	}
	
	private List getInitialParameters(Method method) {
       List list = new ArrayList();
		for (int i = 0; i < method.getArguments().size(); i++ ) {
			VariableDeclaration variable = (VariableDeclaration) method.getArguments().get(i);
			list.add(variable);
		}
		return list;			
	}

	// clean and fills the parameter table widget
	private void fillParameterVariableTable(){
		table.removeAll();
		for ( int i = 0; i < parameterList.size(); i++ ) {
			VariableDeclaration variable = (VariableDeclaration) parameterList.get( i );
			TableItem item = new TableItem( table, SWT.NONE );
			item.setText( new String[] { variable.getName(), variable.getType()});
			item.setData(variable);
		}
	}

}
