/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.dialog;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;


import VOCLModel.*;

/**
 * @author tugstugi, manaljav, bjoern , jan , stefan und der rest der welt, die irgendwannmal daran denken werden....
 * JFace based dialog used to edit variable declaration properties 
 */
public class VariableChoosePropertiesDialog extends TitleAreaDialog implements ModifyListener {	
	private Text nameText;
	private Combo varCombo;
	private Constraint constraint;	
	private ClassifierRole classifierRole;
	private Method method;
	private VariableDeclaration variableDeclaration;
	private Composite composite;
	private Composite superComposite;
	private List newTypes = new Vector();
	private String name;
	private String type;
	
	/**
	 * Constructs a new VariableChoosePropertiesDialog.
	 * @param parentShell the parent shell (becomes blocked when this dialog is open)
	 * @param constraint the constraint
	 * @param classifierRole the classifier role
	 * @param method the method
	 * @param variableDeclaration the variable to be edited
	 */
	public VariableChoosePropertiesDialog(Shell parentShell, Constraint constraint, ClassifierRole classifierRole, Method method, VariableDeclaration variableDeclaration){
		super(parentShell);
		setShellStyle( getShellStyle() | SWT.RESIZE );
		this.constraint = constraint;
		this.classifierRole = classifierRole;
		this.method = method;
		this.variableDeclaration = variableDeclaration;
		name  = variableDeclaration.getName()  != null ? variableDeclaration.getName()  : ""; 
		type  = variableDeclaration.getType()  != null ? variableDeclaration.getType()  : "";
	}
	
	/**
	 * Constructs a new VariableChoosePropertiesDialog.
	 * @param parentShell the parent shell (becomes blocked when this dialog is open)
	 * @param constraint the constraint
	 * @param classifierRole the classifier role
	 * @param method the method
	 * @param variableDeclaration the variable to be edited
	 * @param newTypes the list of new types
	 */
	public VariableChoosePropertiesDialog(Shell parentShell, Constraint constraint, ClassifierRole classifierRole, Method method, VariableDeclaration variableDeclaration, List newTypes){
		super(parentShell);
		setShellStyle( getShellStyle() | SWT.RESIZE );
		this.constraint = constraint;
		this.classifierRole = classifierRole;
		this.method = method;
		this.variableDeclaration = variableDeclaration;
		this.newTypes = newTypes;
		name  = variableDeclaration.getName()  != null ? variableDeclaration.getName()  : ""; 
		type  = variableDeclaration.getType()  != null ? variableDeclaration.getType()  : "";
		nameText = null;
	}

	/**
	 * @author bjoern
	 * 
	 * Helper function, adding a Label with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Label addLabelToComposite( Composite composite, String text ) {
		Label label = new Label( composite, 0 );
		label.setText( text );
		GridData gd = new GridData(); gd.widthHint = 100;
		label.setLayoutData( gd );
		return label;
	}

	/**
	 * @author bjoern
	 * 
	 * Helper function, adding a Text (-field) with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Text addTextToComposite( Composite composite, String textValue, ModifyListener textListener ) {
		Text text = new Text( composite, SWT.SINGLE | SWT.BORDER );
		text.setText( textValue );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		text.setLayoutData( gd );
		if ( textListener != null )
			text.addModifyListener( textListener );
		return text;
	}
	
	
	/**
	 * Helper function, adding a Combo with the GridLayout-Date to a Composite.
	 *
	 * @param composite The composite to add the label to
	 * @param textValue The text of the label to be added to the composite
	 * @return the combo that was added to composite
	 */	
	Combo addComboToCompositeVar(Composite composite, String textValue) {		
		Combo com = new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE | SWT.READ_ONLY);
   	    com.setText( textValue );
   	    GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		com.setLayoutData( gd );
		com.addModifyListener( this );
		List variables = constraint.getAllVariables();
		Iterator it = variables.iterator();
		while(it.hasNext()){
			Object next = it.next();
			if(next instanceof VariableDeclaration) {
				VariableDeclaration var = (VariableDeclaration)next;
				String name = var.getName();
				if ( name != null && ! "".equals( name ) ){
					if(!existsVariable(com,var)){
						com.add(((VariableDeclaration)next).getName()+":"+((VariableDeclaration)next).getType());
					}
					
				}
			}
		}
		return com;
	}
	
	private boolean existsVariable(Combo combo,VariableDeclaration var){
		if(combo.indexOf(var.getName()+":"+var.getType())!=-1)
			return true;
		else 
			return false;
	}
	
	/** 
	 * Creates the content of the dialog.
	 */
	protected Control createDialogArea(Composite parent) {
		superComposite = (Composite) super.createDialogArea(parent);
		composite = new Composite( superComposite, 0 );
		composite.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout( layout );
		
		addLabelToComposite( composite, "Existing Variables:" );
		varCombo  = addComboToCompositeVar( composite,"");		
		
		return superComposite;
	}

	/**
	 * Implementation of ModifyListener.modifyText() used to show error messages and
	 * disable ok button on incomplete input 
	 */
	public void modifyText(ModifyEvent e) {
		
		setErrorMessage( null );
		setMessage( "danke!" );
		getButton(IDialogConstants.OK_ID).setEnabled( true );
		
		String neuVariableName = "";
		if(varCombo.getText().indexOf(":") != -1){
			name = (varCombo.getText()).substring(0,(varCombo.getText()).indexOf(":"));
			type = (varCombo.getText()).substring((varCombo.getText()).indexOf(":")+1,(varCombo.getText()).length());
			
		}else{
			name = varCombo.getText();
			type = "";
		}
		for(int i=0; i<constraint.getAllVariables().size(); i++){
			if(name.equals(((VariableDeclaration) constraint.getAllVariables().get(i)).getName())){
				variableDeclaration = (VariableDeclaration) constraint.getAllVariables().get(i);
				break;
			}			
		}		
	}

	
	/**
	 * Creates contents.
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		Control superContent = super.createContents(parent);		
		parent.getShell().setMinimumSize( 400, 300 );
		parent.getShell().setText( "VariableDeclaration Properties" );
		return superContent;
	}
	
	private boolean existsWhiteSpace(String name){		
		String att = name;
		for(int j=0; j<att.length(); j++){
		 char c = att.charAt(j);
		 if(Character.isWhitespace(c))
			return true;
		}	
		return false;
	}
	
	/**
	 * Cancel pressed.
	 * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
	 */
	protected void cancelPressed() {
		super.cancelPressed();
	}
	
	/**
	 * Sets the edited variableDeclaration properties after user clicks "OK".
	 */
	protected void okPressed() {
		super.okPressed();
	}
	
	/**
	 * Returns the name.
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Sets the type.
	 * @return the type
	 */
	public String getType() {
		return type;
	}
}
