package vocleditor.edit;


import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.views.properties.IPropertySource;
import vocleditor.model.EObjectPropertySource;


/**
 * @author bjoern
 *
 *  AbstractVOCLEditPart is an AbstractGraphicalEditPart that implements
 *  org.eclipse.emf.common.notify.Adapter and manages a shell reference.
 */
public abstract class AbstractVOCLEditPart extends AbstractGraphicalEditPart implements Adapter
{
	Notifier adapterTarget = null;
	Shell    shell = null;

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier target) 
	{
		adapterTarget = target;
	}
	
	
	/**
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return adapterTarget;
	}
	
	/** 
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object arg0) {
		return false;
	}

	
	/**
	 * This method is called on model changes 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public abstract void notifyChanged(Notification notification ); 

	/**
	 * starts listening for model changes
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	public void activate() {
		if (isActive())
			return;
		// start listening for changes in the model
		((Notifier)getModel()).eAdapters().add(this);
		super.activate();
	}
	
	/**
	 * stops listening for model changes
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	public void deactivate() {
		if (!isActive())
			return;
		// stop listening for changes in the model
		((Notifier)getModel()).eAdapters().remove(this);

		super.deactivate();		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
        if (IPropertySource.class == key) {
			return new EObjectPropertySource( (EObject) getModel() );
		}
		return super.getAdapter( key );		
	}
	
	
	/**
	 * sets the shell for this EditPart 
	 * @param shell a shell   
	 */
	public void setShell( Shell shell )
	{
		this.shell = shell;
	}
	
	/**
	 * returns the shell for this EditPart 
	 * @return shell a shell   
	 */
	public Shell getShell()
	{
		return shell;
	}
	
}
