/*
 * Created on May 5, 2005
 */
package vocleditor.edit;



import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartListener;
import org.eclipse.gef.EditPolicy;

import VOCLModel.*;
import vocleditor.VOCLEditorPlugin;
import vocleditor.commands.EditAttributeCommand;
import vocleditor.dialog.AttributePropertiesDialog;
import vocleditor.figures.AttributeFigure;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;

/**
 * EditPart for an Attribute
 * 
 * @author bjoern
 */
public class AttributeEditPart extends AbstractVOCLEditPart implements EditPartListener{
	
	public AttributeEditPart( Attribute attribute )	{
		setModel( attribute );
		addEditPartListener( this );
	}
	
	/**
	 * Creates an {@link AttributeFigure}, initializes an returns it
	 * 
	 * @return the Figure for this EditPart
	 */
	protected IFigure createFigure() {
		Attribute attr = (Attribute) getModel();		
		AttributeFigure afig = new AttributeFigure();
		String text = attr.getName() + " = ";
		if(attr.getValue() != null && !"".equals(attr.getValue())){
			text = text + attr.getValue() + " : ";
		}else if(attr.getVariable() != null) {
			text = text + attr.getVariable().getName() + " : ";
		}
		text = text + attr.getType();
		afig.setText(text);
		return afig;
	}
	
	/**
	 * Installs an {@link AttributeComponentEditPolicy}
	 */
	protected void createEditPolicies() {
		installEditPolicy( EditPolicy.COMPONENT_ROLE, new AttributeComponentEditPolicy() );
	}

	/**
	 * Refreshes the figure after a model change.
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(Notification arg0) {
		Attribute 		attr = (Attribute) getModel(); 
		AttributeFigure afig = (AttributeFigure) getFigure();
		String text = attr.getName() + " = ";
		if(attr.getValue() != null && !"".equals(attr.getValue())){
			text = text + attr.getValue() + " : ";
		}else if(attr.getVariable() != null) {
			text = text + attr.getVariable().getName() + " : ";
		}		
		((VOCLClassifierRoleEditPart) getParent()).refreshVisuals();

		text = text + attr.getType();
		afig.setText(text);
		refreshVisuals();
	}	
	
	/**
	 * Does nothing but is need for {@link EditPartListener}
	 */
	public void childAdded(EditPart child, int index) {}

	/**
	 * Does nothing but is need for {@link EditPartListener}
	 */
	public void partActivated(EditPart editpart) {}

	/**
	 * Does nothing but is need for {@link EditPartListener}
	 */
	public void partDeactivated(EditPart editpart) {}
	
	/**
	 * Does nothing but is need for {@link EditPartListener}
	 */
	public void removingChild(EditPart child, int index) {}

	/**
	 * (Un-)highlights the AttributeFigure if the parts selection state changes. 
	 * @see org.eclipse.gef.EditPartListener#selectedStateChanged(org.eclipse.gef.EditPart)
	 */
	public void selectedStateChanged(EditPart editpart) {
		((AttributeFigure) getFigure()).setSelected( getSelected() );
		refreshVisuals();
	}

	/**
	 * Opens the properties dialog and creates and executes an
	 * {@link EditAttributeCommand} if necessary
	 */
	public void performRequest(Request request) {
		if(request.getType() == RequestConstants.REQ_OPEN) {
			Command command = null;
			
			EditPart editpart = this;
			VOCLClassifierRoleEditPart editpartParent = (VOCLClassifierRoleEditPart) getParent();
			command = new EditAttributeCommand();
			Attribute att = (Attribute)editpart.getModel();
			((EditAttributeCommand)command).setAttribute(att);
			((EditAttributeCommand)command).setParent((ClassifierRole) editpartParent.getModel());
			AttributePropertiesDialog apd = new AttributePropertiesDialog( new Shell(), VOCLEditorPlugin.getConstraint(editpartParent.getClassifierRole()), editpartParent.getClassifierRole(), att);
			apd.open();
			if( apd.getReturnCode() == Dialog.CANCEL)
				return;
			VariableDeclaration variable = null;
			((EditAttributeCommand)command).setCommandAttributes(apd.getName(), apd.getType(), apd.getValue(), apd.getVariableName());

			getRoot().getViewer().getEditDomain().getCommandStack().execute(command);
		}
	}

}
