/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;

import vocleditor.VOCLEditorConstants;
import vocleditor.commands.EditBoolExpCommand;
import vocleditor.commands.EditIfExpCommand;
import vocleditor.dialog.BoolExpPropertiesDialog;
import vocleditor.figures.NegatableBorder;
import vocleditor.figures.SplitFigure;
import vocleditor.model.VOCLTools;
import VOCLModel.BoolExp;

/**
 * EditPart for a Boolean Expression.
 * @author bjoern 
 */
public class BoolExpEditPart extends ExpEditPart
{
	
	/**
	 * Creates a new VOCLEditPart instance.
	 * @param boolexp the {@link BoolExp} to create the EditPart for
	 */
	public BoolExpEditPart( BoolExp boolexp ) {
		if ( boolexp.getWidth() == 0 )
			boolexp.setWidth( 200 );
		if ( boolexp.getHeight() == 0 )
			boolexp.setHeight( 200 );
		setModel( boolexp );
	}

	/**
	 * Returns the model children.
	 * @return the model children.
	 */
	protected List getModelChildren(){
 		Vector children = new Vector();
 		BoolExp boolexp = (BoolExp) getModel();
 		if ( boolexp.getOp1() != null ) {
 			children.add( boolexp.getOp1() );
 		}
 		if ( boolexp.getOp2() != null )
 			children.add( boolexp.getOp2() );
 		return children;
	}
	/**
	 * This method is called on model changes 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(Notification notification) {		
		refreshChildren();
		refreshVisuals();
	}

	protected void createEditPolicies() {
		installEditPolicy( EditPolicy.COMPONENT_ROLE, new BoolExpComponentEditPolicy() );
	}

	protected void refreshVisuals() {
		BoolExp boolexp = (BoolExp) getModel();
		SplitFigure figure = (SplitFigure) getFigure();
		Point loc = new Point( boolexp.getX(), boolexp.getY() );
		Dimension size = new Dimension( boolexp.getWidth(), boolexp.getHeight() );
		Rectangle r = new Rectangle(loc ,size);
		((GraphicalEditPart) getParent()).setLayoutConstraint( this, figure, r);
		figure.setHorizontal( boolexp.isSplitHorizontal() );
		figure.setSplitPercentage( boolexp.getSplitPercent() );
		figure.setMiddleTitle( boolexp.getKind().getName() );
		((NegatableBorder) figure.getBorder()).setNegated( boolexp.isNegated() );
	}

	protected IFigure createFigure() {
		SplitFigure figure;
		BoolExp boolexp = (BoolExp) getModel();

		if ( (VOCLTools.getExpressionDepth( boolexp ) & 1) == 0 )
			figure = new SplitFigure( false, VOCLEditorConstants.EXPRESSION_COLOR_2, VOCLEditorConstants.EXPRESSION_COLOR);
		else
			figure = new SplitFigure( false, VOCLEditorConstants.EXPRESSION_COLOR, VOCLEditorConstants.EXPRESSION_COLOR_2);

		figure.setHorizontal( boolexp.isSplitHorizontal() );
		figure.setSplitPercentage( boolexp.getSplitPercent() );
		((NegatableBorder) figure.getBorder()).setNegated( boolexp.isNegated() );
		return figure;
	}
	
	
	/**
	 * Opens the properties dialog and creates and executes an
	 * {@link EditIfExpCommand} if necessary
	 */
	public void performRequest(Request request) {
		if(request.getType() == RequestConstants.REQ_OPEN) {
			Command command = null;
			AbstractVOCLEditPart editpart = this;
			BoolExp boolexp = (BoolExp)editpart.getModel();
			command = new EditBoolExpCommand( boolexp );
			BoolExpPropertiesDialog bpd = new BoolExpPropertiesDialog( new Shell(), boolexp );
			bpd.open();
			if( bpd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditBoolExpCommand)command).setCommandAttributes(  bpd.getOperation(), bpd.getIsHorizontal(), bpd.getSplitPercentage(), bpd.isNegated() );

			getRoot().getViewer().getEditDomain().getCommandStack().execute(command);
		}
	}
}
