/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartListener;
import org.eclipse.gef.EditPolicy;

import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.NavExp;
import vocleditor.VOCLEditorPlugin;
import vocleditor.commands.EditConditionCommand;
import vocleditor.dialog.ConditionPropertiesDialog;
import vocleditor.figures.ConditionFigure;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;

/**
 * The condition edit part.
 *
 * @author manaljav
 */
public class ConditionEditPart extends AbstractVOCLEditPart implements EditPartListener {
	
	/**
	 * The constructor.
	 * @param condition
	 */
	public ConditionEditPart( Condition condition )	{
		setModel( condition );
		addEditPartListener( this );
		notifyChanged(null);
	}
	
	/**
	 * Creates a {@link ConditionFigure} and sets it text to
	 * {@link #getConditionAsString()}
	 */
	protected IFigure createFigure() {	
		ConditionFigure figure = new ConditionFigure();
		figure.setText(getConditionAsString());
		return figure;
	}

	/**
	 * installs {@link ConditionComponentEditPolicy}
	 */
	protected void createEditPolicies() {
		installEditPolicy( EditPolicy.COMPONENT_ROLE, new ConditionComponentEditPolicy() );
	}
	
	/**
	 * Creates a string describing the condition
	 * @return the Condition as String
	 */
	private String getConditionAsString(){
		Condition condition = (Condition) getModel();
		String text = "";
		if(condition.getLeft() != null)
			text = text + condition.getLeft().getName();
		else{
			text = text + "undefined";
		} 
		if((condition.getLeft()!=null)&&(condition.isLeftPre()))
			text = text + "@pre";
		text = text + " " + condition.getOp();
		if(condition.getRight() != null){
			text = text + " " + condition.getRight().getName();
			if(condition.isRightPre())
				text = text + "@pre";
		}else{
			if(condition.getValue()!=null)
				text = text + " " + condition.getValue();
			else{
				text = text+"undefined";
			}
		}
		return text;
	}

	/**
	 * Refreshes the figure after a model change.
	 */
	public void notifyChanged(Notification note) {
		Condition condition = (Condition) getModel();
		if(condition.getLeft() != null && !condition.getLeft().eAdapters().contains(this))
			condition.getLeft().eAdapters().add(this);
		if(condition.getRight() != null && !condition.getRight().eAdapters().contains(this))
			condition.getRight().eAdapters().add(this);
		ConditionFigure figure = (ConditionFigure) getFigure();
		figure.setText(getConditionAsString());
		refreshVisuals();
	}	
	
	/**
	 * only for inheritance
	 */
	public void childAdded(EditPart child, int index) {}
	/**
	 * only for inheritance
	 */
	public void partActivated(EditPart editpart) {}
	/**
	 * only for inheritance
	 */
	public void partDeactivated(EditPart editpart) {}
	/**
	 * only for inheritance
	 */
	public void removingChild(EditPart child, int index) {}	

	/**
	 * (Un-)Highlights the AttributeFigure if the parts selection state changes.
	 */
	public void selectedStateChanged(EditPart editpart) {
		((ConditionFigure) getFigure()).setSelected( getSelected() );
		refreshVisuals();
	}
	
	/**
	 * Opens the properties dialog and creates and executes an
	 * {@link EditConditionCommand} if necessary
	 */
	public void performRequest(Request request) {
		if(request.getType() == RequestConstants.REQ_OPEN) {
			Command command = null;
			AbstractVOCLEditPart editpart = (ConditionEditPart) this;
			command = new EditConditionCommand();
			Condition condition = (Condition)editpart.getModel();
			((EditConditionCommand)command).setCondition(condition);			
			NavExp navExp = (NavExp) ((NavExpEditPart) editpart.getParent()).getModel();
			Constraint constraint = VOCLEditorPlugin.getConstraint(navExp);
			ConditionPropertiesDialog cpd = new ConditionPropertiesDialog( new Shell(), condition , constraint, navExp);
			cpd.open();
			if( cpd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditConditionCommand)command).setCommandAttributes(cpd.getOp(),cpd.getValue(),cpd.getLeft(),cpd.getRight(),cpd.getLeftPre(),cpd.getRightPre());

			getRoot().getViewer().getEditDomain().getCommandStack().execute(command);
		}
	}
}
