/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;

import VOCLModel.IfExp;
import vocleditor.commands.EditIfExpCommand;
import vocleditor.dialog.IfExpPropertiesDialog;
import vocleditor.figures.IfExpFigure;
import vocleditor.figures.NegatableBorder;
import vocleditor.model.VOCLTools;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;

import vocleditor.VOCLEditorConstants;

/**
 * EditPart for a Boolean Expression.
 * @author bjoern 
 */
public class IfExpEditPart extends ExpEditPart
{
	
	/**
	 * Creates a new VOCLEditPart instance.
	 * @param ifexp the {@link IfExp} to create the EditPart for
	 */
	public IfExpEditPart( IfExp ifexp ) {
		if ( ifexp.getWidth() == 0 )
			ifexp.setWidth( 200 );
		if ( ifexp.getHeight() == 0 )
			ifexp.setHeight( 200 );
		setModel( ifexp );
	}

	/**
	 * Returns the model children.
	 * @return the model children.
	 */
	protected List getModelChildren(){
 		Vector children = new Vector();
 		IfExp ifexp = (IfExp) getModel();
 		if ( ifexp.getIfExp() != null )
 			children.add( ifexp.getIfExp() );
 		if ( ifexp.getThenExp() != null )
 			children.add( ifexp.getThenExp() );
 		if ( ifexp.getElseExp() != null )
 			children.add( ifexp.getElseExp() );
 		return children;
	}

	public void notifyChanged(Notification notification) {		
		refreshChildren();
		refreshVisuals();
	}

	protected void createEditPolicies() {
		installEditPolicy( EditPolicy.COMPONENT_ROLE, new IfExpComponentEditPolicy() );
	}

	protected void refreshVisuals() {
		IfExp ifexp = (IfExp) getModel();
		IfExpFigure figure = (IfExpFigure) getFigure();
		Point loc = new Point( ifexp.getX(), ifexp.getY() );
		Dimension size = new Dimension( ifexp.getWidth(), ifexp.getHeight() );
		Rectangle r = new Rectangle(loc ,size);
		((GraphicalEditPart) getParent()).setLayoutConstraint( this, figure, r);
		figure.setHorizontal( ifexp.isSplitHorizontal() );
		figure.setSplitPercentage1( ifexp.getSplitPercent1() );
		figure.setSplitPercentage2( ifexp.getSplitPercent2() );
		((NegatableBorder) figure.getBorder()).setNegated( ifexp.isNegated() );
	}

	protected IFigure createFigure() {
		IfExp ifexp = (IfExp) getModel();
		IfExpFigure figure;

		if ( (VOCLTools.getExpressionDepth( ifexp ) & 1) == 0 )
			figure = new IfExpFigure( VOCLEditorConstants.EXPRESSION_COLOR_2, VOCLEditorConstants.EXPRESSION_COLOR);
		else
			figure = new IfExpFigure( VOCLEditorConstants.EXPRESSION_COLOR, VOCLEditorConstants.EXPRESSION_COLOR_2);
		figure.setHorizontal( ifexp.isSplitHorizontal() );
		figure.setSplitPercentage1( ifexp.getSplitPercent1() );
		figure.setSplitPercentage2( ifexp.getSplitPercent2() );
		((NegatableBorder) figure.getBorder()).setNegated( ifexp.isNegated() );
		return figure;
	}

	/**
	 * Opens the properties dialog and creates and executes an
	 * {@link EditIfExpCommand} if necessary
	 */
	public void performRequest(Request request) {
		if(request.getType() == RequestConstants.REQ_OPEN) {
			Command command = null;
			AbstractVOCLEditPart editpart = (IfExpEditPart) this;
			IfExp ifexp = (IfExp)editpart.getModel();
			command = new EditIfExpCommand( ifexp );
			IfExpPropertiesDialog ipd = new IfExpPropertiesDialog( new Shell(), ifexp );
			ipd.open();
			if( ipd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditIfExpCommand)command).setCommandAttributes(  ipd.getIsHorizontal(), ipd.getSplitPercentage1(), ipd.getSplitPercentage2(), ipd.isNegated() );

			getRoot().getViewer().getEditDomain().getCommandStack().execute(command);
		}
	}
}
