/*
 * Created on 21.06.2005
 *
 */
package vocleditor.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import VOCLModel.AssociationRole;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.draw2d.PolylineConnection;

import vocleditor.commands.MoveAssociationRoleLabelCommand;

/**
 * This is the EditPart for the AssociationRole's Labels
 * @author jan
 */
public class LabelGraphicalEditPart extends AbstractGraphicalEditPart {
	/**
	 * The Label's Figure
	 */ 
	protected Label figure;
	
	/**
	 * The associated AssociationRole
	 */
	AssociationRole parent;
	
	/**
	 * Constructor, sets the @see LabelGraphicalEditPart#parent
	 * @param parent The associated AssociationRole
	 */
	LabelGraphicalEditPart(AssociationRole parent) {
		this.parent = parent;
	}
	
	/**
	 * Creates the Figure if it doesn't exist yet. Also handles the endpointlocators.
	 */
	protected IFigure createFigure() {
		if(figure == null) {
			figure = new Label(parent.getName());
		}
		
		PolylineConnection con = (PolylineConnection)((VOCLAssociationRoleEditPart)getParent()).getConnectionFigure();
		VOCLConnectionEndpointLocator loc = new VOCLConnectionEndpointLocator(con, false, new Point(parent.getLabelDecoration().getX(), parent.getLabelDecoration().getY()), parent.getLabelDecoration().isCustomPosition());
		if(figure.getParent() != null)
			figure.getParent().setConstraint(figure, loc);
		
		figure.setText(parent.getName());
		return figure;
	}

	/** 
	 * Simply calls {@link #createFigure()}. All necessary work is done there.
	 */
	public IFigure getFigure() {
		return createFigure();
	}
	
	/**
	 * Applies selection to the parent connection, too.
	 * @see AbstractGraphicalEditPart#setSelected
	 */
	public void setSelected(int i) {
		if(this.getParent() != null)
			this.getParent().setSelected(i);
		super.setSelected(i);
	}
	
	/**
	 * Installs edit policies.
	 * @see AbstractGraphicalEditPart#createEditPolicies
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new NonResizableEditPolicy());
	}

	/**
	 * Returns the MoveCommand if necessary. Forwards all other requests to the parent (For deletion etc.)
	 * There is some kind of hack here, we set the VOCLAssociationRoleEditPart parent to NavExpEditPart for Dragging. If we don't do that here
	 * @link DragEditPartsTracker#isMove() will always return false. This seems in no way disturb anything else.
	 */
	public Command getCommand(Request request) {
		if(request instanceof ChangeBoundsRequest) {
			if(this.getParent() != null) {
				getParent().setParent(((VOCLAssociationRoleEditPart)getParent()).layerEditPart);
			}
			Point delta = ((ChangeBoundsRequest)request).getMoveDelta();
			Point connectionEndPoint = Point.SINGLETON;
			((VOCLAssociationRoleEditPart)getParent()).getConnectionFigure().getPoints().getPoint(connectionEndPoint, 0);
 
			MoveAssociationRoleLabelCommand command = new MoveAssociationRoleLabelCommand(parent, 
					this.getFigure(), 
					delta,
					connectionEndPoint.getCopy()
			);
			return command;
		}
		return getParent().getCommand(request); // Will get DeleteCommand etc. from parent.
	}

	/**
	 * Reacts on double-clicks. Forwards the request to the parent associationrole's
	 * EditPart.
	 * @see VOCLAssociationRoleEditPart#performRequest(Request)
	 */
	public void performRequest(Request request) {
		if(request.getType() == RequestConstants.REQ_OPEN) {
			getParent().performRequest(request);
		}	
	}

}
