/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;

import VOCLModel.LetExp;
import VOCLModel.VariableDeclaration;
import vocleditor.commands.EditLetExpCommand;
import vocleditor.dialog.LetExpPropertiesDialog;
import vocleditor.figures.NegatableBorder;
import vocleditor.figures.SplitFigure;
import vocleditor.model.VOCLTools;

import vocleditor.VOCLEditorConstants;
import vocleditor.VOCLEditorPlugin;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;

/**
 * EditPart for a Boolean Expression.
 * @author bjoern 
 */
public class LetExpEditPart extends ExpEditPart
{
	
	/**
	 * Creates a new VOCLEditPart instance.
	 * @param letexp the {@link LetExp} to create the EditPart for
	 */
	public LetExpEditPart( LetExp letexp ) {
		if ( letexp.getWidth() == 0 )
			letexp.setWidth( 200 );
		if ( letexp.getHeight() == 0 )
			letexp.setHeight( 200 );
		setModel( letexp );	
		VOCLEditorPlugin.getConstraint( letexp.getParentExp() ).eAdapters().add( this );
	}

	/**
	 * Returns the model children.
	 * @return the model children.
	 */
	protected List getModelChildren(){
 		Vector children = new Vector();
 		LetExp letexp = (LetExp) getModel();
 		if ( letexp.getLet() != null ) {
 			children.add( letexp.getLet() );
 		}
 		if ( letexp.getIn() != null )
 			children.add( letexp.getIn() );
 		return children;
	}
	/**
	 * This method is called on model changes 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(Notification notification) {
		LetExp letexp = (LetExp) getModel();
		VariableDeclaration var = letexp.getLetVar();
		if ( var != null ) {
			if( ! var.eAdapters().contains( this ) )
				var.eAdapters().add( this );
		}
		refreshChildren();
		refreshVisuals();
	}

	protected void createEditPolicies() {
		installEditPolicy( EditPolicy.COMPONENT_ROLE, new LetExpComponentEditPolicy() );
	}

	protected void refreshVisuals() {
		LetExp letexp = (LetExp) getModel();
		GraphicalEditPart parentEP = (GraphicalEditPart) getParent();
		
		if ( parentEP == null ) return; // happens when deleting smthng

		SplitFigure figure = (SplitFigure) getFigure();
		Point loc = new Point( letexp.getX(), letexp.getY() );
		Dimension size = new Dimension( letexp.getWidth(), letexp.getHeight() );
		Rectangle r = new Rectangle(loc ,size); 
		parentEP.setLayoutConstraint( this, figure, r);
		figure.setHorizontal( letexp.isSplitHorizontal() );
		figure.setSplitPercentage( letexp.getSplitPercent() );
		if ( letexp.getLetVar() != null )
			figure.setTopLabel( "" + letexp.getLetVar().getName() + " : " + letexp.getLetVar().getType() );
		else
			figure.setTopLabel( "no variable" );
		((NegatableBorder) figure.getBorder()).setNegated( letexp.isNegated() );
	}

	protected IFigure createFigure() {
		SplitFigure figure;
		LetExp letexp = (LetExp) getModel();

		if ( (VOCLTools.getExpressionDepth( letexp ) & 1) == 0 )
			figure = new SplitFigure( false, VOCLEditorConstants.EXPRESSION_COLOR_2, VOCLEditorConstants.EXPRESSION_COLOR);
		else
			figure = new SplitFigure( false, VOCLEditorConstants.EXPRESSION_COLOR, VOCLEditorConstants.EXPRESSION_COLOR_2);
			
		figure.setHorizontal( letexp.isSplitHorizontal() );
		figure.setSplitPercentage( letexp.getSplitPercent() );
		figure.setTopTitle( "let" );
		figure.setMiddleTitle( "in" );
		((NegatableBorder) figure.getBorder()).setNegated( letexp.isNegated() );
		return figure;
	}

	/**
	 * Opens the properties dialog and creates and executes an
	 * {@link EditLetExpCommand} if necessary
	 */
	public void performRequest(Request request) {
		if(request.getType() == RequestConstants.REQ_OPEN) {
			Command command = null;
			AbstractVOCLEditPart editpart = (LetExpEditPart) this;
			LetExp letexp = (LetExp)editpart.getModel();
			command = new EditLetExpCommand( letexp );
			LetExpPropertiesDialog bpd = new LetExpPropertiesDialog( new Shell(), letexp );
			bpd.open();
			if( bpd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditLetExpCommand)command).setCommandAttributes( bpd.getIsHorizontal(), bpd.getSplitPercentage(), bpd.getVariableDeclaration(), bpd.isNegated() );

			getRoot().getViewer().getEditDomain().getCommandStack().execute(command);
		}
	}
}
