/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
//import org.eclipse.swt.widgets.Shell;

import VOCLModel.AssociationRole;
import VOCLModel.ClassifierRole;
import vocleditor.commands.CreateAssociationRoleCommand;
import vocleditor.commands.ReconnectAssociationRoleCommand;


/**
 * EditPolicy for a Connection Role.
 */
public class VOCLClassifierRoleEditPolicy 
	extends GraphicalNodeEditPolicy {


	public VOCLClassifierRoleEditPolicy() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request) {
//		Shell shell = ((VOCLClassifierRoleEditPart)getHost()).getShell();
		Command command = request.getStartCommand();
		if (command instanceof CreateAssociationRoleCommand) {
			CreateAssociationRoleCommand co = (CreateAssociationRoleCommand)command;
			ClassifierRole target = getVOCLModelClassifierRole();
			// dont connect classifiers from different constraints
			if ( target.getNavExp() == co.getSource().getNavExp() ) {
				co.setTarget( target );
//				co.setShell(shell);
				return co;
			}
		}
		return null;
	}
	
	protected ClassifierRole getVOCLModelClassifierRole() {
		return (ClassifierRole)getHost().getModel();
	}
	
	protected VOCLClassifierRoleEditPart getVOCLClassifierRoleEditPart() {
		return (VOCLClassifierRoleEditPart)getHost();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		CreateAssociationRoleCommand command = new CreateAssociationRoleCommand();
		Object reqObj = request.getNewObject();
		if (reqObj instanceof AssociationRole) {
			command.setAssociationRole((AssociationRole)reqObj);
		}
		command.setSource(getVOCLModelClassifierRole());
		request.setStartCommand(command);
		return command;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectTargetCommand(ReconnectRequest request) 
	{
		AssociationRole connection = (AssociationRole) request.getConnectionEditPart().getModel();	
		ClassifierRole source = (ClassifierRole)request.getTarget().getModel();
		ClassifierRole target = connection.getAssociationTarget();

		if ( target.getNavExp() == source.getNavExp() ) {
			ReconnectAssociationRoleCommand command = new ReconnectAssociationRoleCommand(connection, request.isMovingStartAnchor());
			command.setSource(source);
			command.setAssociationRole(connection);
			command.setTarget( target );
			command.setTarget_old(connection.getAssociationTarget());
			command.setSource_old(connection.getAssociationSource());
			command.setEdPart((VOCLAssociationRoleEditPart)request.getConnectionEditPart());
			return command;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectSourceCommand(ReconnectRequest request) {
		AssociationRole connection = (AssociationRole) request.getConnectionEditPart().getModel();
		ClassifierRole target = (ClassifierRole) request.getTarget().getModel();
		ClassifierRole source = connection.getAssociationSource();
		
		if ( target.getNavExp() == source.getNavExp() ) {	
			ReconnectAssociationRoleCommand command = new ReconnectAssociationRoleCommand(connection, request.isMovingStartAnchor());
			command.setTarget( target );
			command.setAssociationRole(connection);
			command.setSource( source );
			command.setTarget_old(connection.getAssociationTarget());
			command.setSource_old(connection.getAssociationSource());
			command.setEdPart((VOCLAssociationRoleEditPart)request.getConnectionEditPart());
			return command;
		}
		return null;
	}
}
