/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;

import VOCLModel.Constraint;
import VOCLModel.VOCLModelPackage;
import vocleditor.figures.ConstraintFigure;

import vocleditor.VOCLEditorPlugin;

/**
 * EditPart for a Constraint. 
 */
public class VOCLConstraintEditPart extends AbstractVOCLEditPart{
	
	/**
	 * Creates a new VOCLEditPart instance.
	 * @param constraint the constraint
	 */
	public VOCLConstraintEditPart( Constraint constraint ) {
		VOCLEditorPlugin.printDebug( "NEW VOCLConstraintEditPart");
		setModel( constraint );
	}

	/**
	 * Returns the model children.
	 * @return the model children.
	 */
	protected List getModelChildren(){
 		Vector children = new Vector();
 		if (((Constraint) getModel()).getContext() != null )
 	 		children.add( ((Constraint) getModel()).getContext() );
 		if (((Constraint) getModel()).getExpression() != null )
 			children.add( ((Constraint) getModel()).getExpression() );
 		return children;
	}

	public void notifyChanged(Notification notification) {		
		int featureId = notification.getFeatureID( VOCLModelPackage.class );
		if ( notification.getEventType() == Notification.ADD ) {
			refreshChildren();
		}
		if ( notification.getEventType() == Notification.REMOVE ) {
			refreshChildren();
		}		
		refreshChildren();
		refreshVisuals();
	}

	protected void createEditPolicies() {
		// install the edit policy to handle deletion
		installEditPolicy( EditPolicy.COMPONENT_ROLE, new ConstraintComponentEditPolicy() );
	}

	protected void refreshVisuals() {
		VOCLEditorPlugin.printDebug( "VOCLCONSTRAINTEPART.REFRESH_VISUALS");
		Constraint constraint = (Constraint) getModel();			
		Point loc = new Point( constraint.getX(), constraint.getY() );
		Dimension size = new Dimension( constraint.getWidth(), constraint.getHeight() );
		Rectangle r = new Rectangle(loc ,size);
		((GraphicalEditPart) getParent()).setLayoutConstraint( this, getFigure(), r);
	}

	protected IFigure createFigure() {
		ConstraintFigure figure = new ConstraintFigure(  );
		return figure;
	}
}
