/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.editor;

import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;

import VOCLModel.AssociationRole;
import VOCLModel.BoolExp;
import VOCLModel.ClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.SetClassifierRole;
import vocleditor.model.ModelCreationFactory;

/**
 * Main class for the VOCL editor palette.
 * All graphical figures have to be registered here.
 */
public class VOCLPaletteRoot extends PaletteRoot
{

	/**
	 * Creates a new VOCLPaletteRoot instance.
	 * 
	 */
	public VOCLPaletteRoot()
	{
		// create root
		super();

		// a group of default control tools
		PaletteGroup controls = new PaletteGroup("Controls");
		add(controls);

		// the selection tool
		ToolEntry tool = new SelectionToolEntry();
		controls.add(tool);

		// use selection tool as default entry
		setDefaultEntry(tool);

		// the marquee selection tool
		controls.add(new MarqueeToolEntry());

		// a group for the editor symbols
		PaletteGroup symbols = new PaletteGroup("Symbols");
		add(symbols);

		ToolEntry entry;
		
		entry =
			new VOCLCreationToolEntry(
				"Constraint",
				"Creates a new constraint",
				new ModelCreationFactory(Constraint.class),
				null,
				null);
		symbols.add(entry);

		entry =
			new VOCLCreationToolEntry(
				"Bool Expression",
				"Creates a new boolean expression",
				new ModelCreationFactory(BoolExp.class),
				null,
				null);
		symbols.add(entry);

		entry =
			new VOCLCreationToolEntry(
				"If Expression",
				"Creates a new if expression",
				new ModelCreationFactory(IfExp.class),
				null,
				null);
		symbols.add(entry);

		entry =
			new VOCLCreationToolEntry(
				"Let Expression",
				"Creates a new let expression",
				new ModelCreationFactory(LetExp.class),
				null,
				null);
		symbols.add(entry);

		entry =
			new VOCLCreationToolEntry(
				"Classifier Role",
				"Creates a classifier role",
				new ModelCreationFactory(ClassifierRole.class),
				null,
				null);
		symbols.add(entry);
		
		entry =
			new VOCLCreationToolEntry(
				"Set Classifier Role",
				"Creates a set classifier role",
				new ModelCreationFactory(SetClassifierRole.class),
				null,
				null);
		symbols.add(entry);
		
		VOCLConnectionCreationToolEntry connectionentry;

 
		entry =
			new VOCLConnectionCreationToolEntry(
				"Association Role ",
				"Creates an association role",
				new ModelCreationFactory(AssociationRole.class),
				null,
				null);
		
		symbols.add(entry);        
	}
}
