/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.CompoundBorder;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Image;
import vocleditor.VOCLEditorConstants;


/**
 * The graphical representation of a VOCL constraint.
 * This resizable graphical representation of a VOCL constraint contains
 * figures for the VOCL model elements Context, Expression and Condition
 * 
 * @author bjoern
 */
public class ConstraintFigure extends RectangleFigure 
{
	protected IFigure  contextFigure;
	protected IFigure exprFigure = null; 
		
	/**
	 * Constructs a new ConstraintFigure
	 *
	 */
	public ConstraintFigure() 
	{
	    setLayoutManager( new ToolbarLayout() );
		setBorder( new CompoundBorder(   // the double-border trick
				new	LineBorder( ColorConstants.black, 1),
				new	LineBorder( VOCLEditorConstants.CONSTRAINT_COLOR, 8 )));
	    setBackgroundColor( VOCLEditorConstants.EXPRESSION_COLOR );
	    setOpaque( true );

	    contextFigure = new Label("no context yet" );
	    ((Label)contextFigure).setIcon( new Image( null, ConstraintFigure.class.getResourceAsStream( "icons/classifier_role.gif" )));
	    add( contextFigure,0 );
	}	

	/**
	 * redefined to add NavExpFigure as second child figure.
	 * @see org.eclipse.draw2d.IFigure#add(org.eclipse.draw2d.IFigure, java.lang.Object, int)
	 */
	public void add(IFigure figure, Object constraint, int index) 
	{	
		if ( figure instanceof ContextFigure && figure != contextFigure ) {
			super.remove( contextFigure );
			contextFigure = figure;
			super.add(figure, 0);
		} else	if ( figure instanceof NavExpFigure && figure != exprFigure ) {
			exprFigure = figure;
			super.add(figure, 1);
		}else
		    
			super.add( figure, constraint, index );
	}
	
	/**
	 * redefined to set Expression figure size
	 * @see org.eclipse.draw2d.IFigure#validate()
	 */
	public void validate(){
		if ( isValid() ) return;

		int efw = getBounds().width - 4;
		int efh = getBounds().height - contextFigure.getBounds().height;
		if (exprFigure instanceof NavExpFigure)
			((NavExpFigure) exprFigure).setPreferredSize( new Dimension( efw, efh ) );
		super.validate();
	}
	
	/**
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}
	
	/**
	 * Returns the minimum size of this ConstraintFigure
	 * based on the size of the context, expression and condition figures.
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) 
	{
		int width, height;

		Dimension contDim = contextFigure.getPreferredSize();
		Dimension condDim = new Dimension(-1, -1); 
		Dimension exprDim = exprFigure.getMinimumSize();

		width  = Math.max( contDim.width + 2, 50 );
		width  = Math.max( condDim.width + 2, width );
		width  = Math.max( exprDim.width + 2, width );
		
		height = Math.max( contDim.height + condDim.height + exprDim.height + 4, 50 );

		return new Dimension( width, height );
	}
}