package vocleditor.figures;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;

/**
 * Help figure class for another figutes.
 * @author bjoern
 */
public class LabeledFigure extends RectangleFigure
{
	LabelBorder labelBorder;

	/**
	 * The default constructor.
	 * @param label the label
	 * @param back the background color
	 * @param outside the outside color
	 */
	LabeledFigure(String label, Color back, Color outside) 
	{
		setLayoutManager(new ToolbarLayout());
		labelBorder = new LabelBorder(back, outside);
		labelBorder.setTitle(label);
		labelBorder.setLabel( "" );
		setBackgroundColor(back);
		setOutline( false );
		labelBorder.setBackgroundColor(getBackgroundColor());
		setBorder( labelBorder );
	}

	/**
	 * redefined to set size if the parts based on whether this figure is split horizontally or not
	 * @see org.eclipse.draw2d.IFigure#validate()
	 */
	public void validate() 
	{
		if ( isValid() ) return;
		super.validate();

		Rectangle bounds = getBounds();
		Insets insets = labelBorder.getInsets( this );
		
		if ( getChildren().size() > 0 )
			((Figure) getChildren().get(0)).setPreferredSize( new Dimension( bounds.width - (insets.left + insets.right), bounds.height - (insets.top + insets.bottom)) );
	}
	
	/**
	 * Returns the minimum size of this Figure
	 * based on the size of the if part and the then-else part.
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint)
	{
		if ( getChildren().size() == 0 )
			return super.getMinimumSize(wHint, hHint);

		Insets insets = labelBorder.getInsets( this );
		
		Dimension d = ((Figure) getChildren().get(0)).getMinimumSize();
		Insets bi = labelBorder.getInsets(this);
		return new Dimension( d.width + bi.left + bi.right, d.height + bi.top + bi.bottom );
	}	
}
