/*
 * Created on 01.06.2005
 * 
 * VOCLEditor PlugIn for Eclipse
 * Gruppe 4 - Sommersemester 2005
 */
package vocleditor.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * @author tugstugi
 * Represents the method.
 */
public class MethodFigure extends RectangleFigure {
	/**
	 * Height of the arrow head.
	 */
	public static int ARROW_HEAD_HEIGHT = 5;
	/**
	 * Width of the arrow head.
	 */
	public static int ARROW_HEAD_WIDTH = 10;
	protected Label nameLabel;
	protected XYLayout layout;
	
	/**
	 * The default constructor.
	 */
	public MethodFigure(){
		layout = new XYLayout();
		setLayoutManager(layout);
		setOutline(false);
		nameLabel = new Label("test");		
		add(nameLabel);
	}
	
	/**
	 * Sets the name of the name label.
	 * @param name the name to set
	 */
	public void setName(String name){
		nameLabel.setText(name);
	}
	
	/**
	 * redefined to set size if the parts based on whether this figure is split horizontally or not
	 * @see org.eclipse.draw2d.IFigure#validate()
	 */
	public void validate(){
		layout.setConstraint(nameLabel, new Rectangle(ARROW_HEAD_WIDTH + ARROW_HEAD_WIDTH/2, ARROW_HEAD_HEIGHT/2 ,-1,-1));
		super.validate();
	}
	
	/**
	 * Paints the figute.
	 * @see org.eclipse.draw2d.Figure#paintFigure()
	 */
	public void paintFigure(Graphics g){
		super.paintFigure(g);
		g.drawLine(getLocation().x + ARROW_HEAD_WIDTH/2, getLocation().y, getLocation().x + ARROW_HEAD_WIDTH/2, getLocation().y + getMinimumSize().height);
		g.drawLine(getLocation().x + ARROW_HEAD_WIDTH/2, getLocation().y + getMinimumSize().height, getLocation().x , getLocation().y + getMinimumSize().height - ARROW_HEAD_HEIGHT);
		g.drawLine(getLocation().x + ARROW_HEAD_WIDTH/2, getLocation().y + getMinimumSize().height, getLocation().x + ARROW_HEAD_WIDTH, getLocation().y + getMinimumSize().height - ARROW_HEAD_HEIGHT);		
	}
	
	/**
	 * Says that this figure uses local coordinates.
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}
	
	/**
	 * Returns the minimum size of this Figure
	 * based on the size of the if part and the then-else part.
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {		
		return getPreferredSize();
	}
	
	/**
	 * returns the preferred size of this figure
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		int height = nameLabel.getPreferredSize().height + ARROW_HEAD_HEIGHT + 5;
		int width = 2*ARROW_HEAD_WIDTH + nameLabel.getPreferredSize().width;
		return new Dimension(width, height);
	}
}
