/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

import vocleditor.VOCLEditorConstants;
import vocleditor.edit.VOCLClassifierRoleEditPart;

/**
 * @author tugstugi
 * Represents the graphical layout of a set classifier role.
 */
public class SetClassifierRoleFigure extends RectangleFigure {	
	public static int SET_SIZE = 5;
	protected XYLayout layout;
	protected ClassifierFigure classifierFigure;
	protected RectangleFigure rectangle;
	protected Label opKindLabel;
	protected String opKind;
	
	/**
	 * The constructor.
	 * @param editPart the classifier role edit part
	 */
	public SetClassifierRoleFigure(VOCLClassifierRoleEditPart editPart) {
		layout = new XYLayout();
	    setLayoutManager(layout);
	    setOutline(false);
	    rectangle = new RectangleFigure();
	    rectangle.setBackgroundColor( VOCLEditorConstants.SET_CLASSIFIER_BACK_COLOR );
	    rectangle.setOpaque( true );
	    this.add(rectangle);
	    
	    classifierFigure = new ClassifierFigure(editPart);
	    this.add(classifierFigure);	    
	    
	    opKindLabel = new Label();
	}
	
	/**
	 * Sets the operation kind.
	 * @param opKind the operation kind
	 */
	public void setOpKind(String opKind, String sizeVariable, String iteratorVariable, String sumVariable){
		this.opKind = opKind;
		if("isEmpty".equals(opKind)){
			opKindLabel.setText("=\u00d8");
		}else if("notEmpty".equals(opKind)){
			opKindLabel.setText("\u2260\u00d8");
		}else if("size".equals(opKind)){
			opKindLabel.setText("#"+sizeVariable);
		}else if("sum".equals(opKind)){
			opKindLabel.setText("\u2211"+iteratorVariable+"="+sumVariable);
		}else{
			opKindLabel.setText(opKind);
		}
		if(opKind != null && !"".equals(opKind)){
			if(!this.getChildren().contains(opKindLabel))
				this.add(opKindLabel);
		}else{
			if(this.getChildren().contains(opKindLabel))
				this.remove(opKindLabel);
		}
		validate();
	}
	
	/**
	 * Returns the classifier role figure.
	 * @return the classifier role figure
	 */
	public ClassifierFigure getClassifierFigure(){
		return classifierFigure;
	}
	
	/**
	 * Validates the figure.
	 */
	public void validate(){
		layout.setConstraint(rectangle, new Rectangle(SET_SIZE, 0, classifierFigure.getPreferredSize().width, classifierFigure.getPreferredSize().height));
		layout.setConstraint(classifierFigure, new Rectangle(0, SET_SIZE ,-1,-1));
		if(opKind != null && !"".equals(opKind))
			layout.setConstraint(opKindLabel, new Rectangle(classifierFigure.getPreferredSize().width + SET_SIZE + 5, classifierFigure.getPreferredSize().height - opKindLabel.getPreferredSize().height - 1, -1, -1));
		super.validate();
	}
	
	/**
	 * Adds a child figure.
	 */
	public void add(IFigure figure, Object constraint, int index){	
		if (figure instanceof AttributeFigure)
			classifierFigure.add(figure, constraint, index);
		else
			super.add(figure, constraint, index);
	}
	
	/**
	 * Removes a child figure.
	 */
	public void remove(IFigure figure) {
		if (figure instanceof AttributeFigure)
			classifierFigure.remove(figure);
		else
			super.remove(figure);
	}
	
	/**
	 * Sets the constrainf of a child figure.
	 */
	public void setConstraint(IFigure figure, Object constraint){
		if (figure instanceof AttributeFigure)
			classifierFigure.setConstraint(figure, constraint);
		else
			super.setConstraint(figure, constraint);
	}
	
	/**
	 * Paints the figure.
	 * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
	 */
	public void paintFigure(Graphics g){
		super.paintFigure(g);
		int style = g.getLineStyle();
		g.setLineStyle(Graphics.LINE_DOT);
		if(opKind != null && !"".equals(opKind)){
			g.drawLine(this.getLocation().x + classifierFigure.getPreferredSize().width + SET_SIZE, this.getLocation().y + classifierFigure.getPreferredSize().height - opKindLabel.getPreferredSize().height, 
					this.getLocation().x + this.getPreferredSize().width, this.getLocation().y + classifierFigure.getPreferredSize().height - opKindLabel.getPreferredSize().height);
			g.drawLine(this.getLocation().x + classifierFigure.getPreferredSize().width + SET_SIZE, this.getLocation().y + classifierFigure.getPreferredSize().height - 1, 
					this.getLocation().x + this.getPreferredSize().width, this.getLocation().y + classifierFigure.getPreferredSize().height - 1);
			g.drawLine(this.getLocation().x + this.getPreferredSize().width - 1, this.getLocation().y + classifierFigure.getPreferredSize().height - opKindLabel.getPreferredSize().height,
					this.getLocation().x + this.getPreferredSize().width - 1, this.getLocation().y + classifierFigure.getPreferredSize().height - 1);
		}
		g.setLineStyle(style);
	}
	
	/**
	 * Says that this figure uses local coordinates.
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}
	
	/**
	 * returns the minimum size of this NavExpFigure
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {		
		return getPreferredSize();
	}
	
	/**
	 * returns the preferred size of this figure
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		int width = classifierFigure.getPreferredSize().width + SET_SIZE;
		if(opKind != null && !"".equals(opKind))
			width = width + opKindLabel.getPreferredSize().width + 6;
		int height = classifierFigure.getPreferredSize().height + SET_SIZE;
		return new Dimension(width, height);
	}
}