/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.figures;


import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Color;


/**
 * The graphical representation of a Boolean Expression.
 * This resizable graphical representation of a VOCL boolExp contains ...
 * TODO
 * 
 * @author bjoern
 */
public class SplitFigure extends RectangleFigure 
{
	ToolbarLayout layout;
	int splitPercentage = 50;
	boolean horizontal = false;
	boolean oppositeTextPos = true;
	LabeledFigure subFigure1, subFigure2;
		
	/**
	 * Constructs a new BoolExpFigure.
	 */
	public SplitFigure( boolean oppositeTextPos, Color fillColor, Color outsideColor ) 
	{
		this.oppositeTextPos = oppositeTextPos;
		layout = new ToolbarLayout(); 
	    setLayoutManager( layout );
	    setBackgroundColor( fillColor );
	    NegatableBorder border = new NegatableBorder();
	    border.setBackground( outsideColor );
		setBorder( border );
		setOpaque( true );
	    setOutline( false );
		subFigure1 = new LabeledFigure( "", fillColor, outsideColor );
		subFigure2 = new LabeledFigure( "", fillColor, fillColor );

		add( subFigure1 );
		add( subFigure2 );
	    layout.setMinorAlignment(ToolbarLayout.ALIGN_TOPLEFT);
	}	

	/**
	 * Sets the horizontal flag.
	 * @param horizontal the horizontal flag
	 */
	public void setHorizontal( boolean horizontal )
	{
		this.horizontal = horizontal;
		layout.setVertical( ! horizontal );
		if ( horizontal ) {
			if ( oppositeTextPos ) {
				subFigure1.labelBorder.setTextPos( PositionConstants.TOP );
				subFigure2.labelBorder.setTextPos( PositionConstants.TOP );
				subFigure1.labelBorder.setLines( true, true, true, false, false, true );
				subFigure2.labelBorder.setLines( true, true, true, true, true, false );
			} else {
				subFigure1.labelBorder.setTextPos( PositionConstants.LEFT );
				subFigure2.labelBorder.setTextPos( PositionConstants.LEFT );
				subFigure1.labelBorder.setLines( true, true, true, false, false, false );
				subFigure2.labelBorder.setLines( true, true, true, true, true, true );
			}
		} else {
			if ( oppositeTextPos ) {
				subFigure1.labelBorder.setTextPos( PositionConstants.LEFT );
				subFigure2.labelBorder.setTextPos( PositionConstants.LEFT );
				subFigure1.labelBorder.setLines( true, true, true, true, true, false );
				subFigure2.labelBorder.setLines( false, true, true, true, false, true );
			} else {
				subFigure1.labelBorder.setTextPos( PositionConstants.TOP );
				subFigure2.labelBorder.setTextPos( PositionConstants.TOP );
				subFigure1.labelBorder.setLines( true, false, true, true, false, false );
				subFigure2.labelBorder.setLines( true, true, true, true, true, true );
			}
		}
		invalidateTree();
		revalidate();
	}

	/**
	 * Sets the split percentage.
	 * @param split the split percentage
	 */
	public void setSplitPercentage( int split )
	{
		splitPercentage = split;
		invalidateTree();
		revalidate();
	}

	/**
	 * Sets the top title.
	 * @param text the title text
	 */
	public void setTopTitle( String text )
	{
		subFigure1.labelBorder.setTitle( text );
	}

	/**
	 * Sets the middle title.
	 * @param text the title text
	 */
	public void setMiddleTitle( String text )
	{
		subFigure2.labelBorder.setTitle( text );
	}

	/**
	 * Sets the top label.
	 * @param text the top label text
	 */
	public void setTopLabel( String text ) 
	{
		subFigure1.labelBorder.setLabel( text );
	}

	/**
	 * Sest the middle label.
	 * @param text the moddle label text
	 */
	public void setMiddleLabel( String text ) 
	{
		subFigure2.labelBorder.setLabel( text );
	}
	
	/**
	 * Adds a figure.
	 * @see org.eclipse.draw2d.IFigure#add(org.eclipse.draw2d.IFigure, java.lang.Object, int)
	 */
	public void add(IFigure figure, Object constraint, int index) 
	{	
		if ( figure == subFigure1 || figure == subFigure2 )
			super.add( figure, constraint, index );
		else if ( subFigure1.getChildren().size() == 0 )
			subFigure1.add( figure, constraint, subFigure1.getChildren().size() );
		else 
			subFigure2.add( figure, constraint, subFigure2.getChildren().size() );		
	}
	
	/**
	 * redefined to set Expression figure size
	 * @see org.eclipse.draw2d.IFigure#validate()
	 */
	public void validate() 
	{
		if ( isValid() ) return;
		super.validate();

		int w = getBounds().width + 1;
		int h = getBounds().height + 1;
		
		if ( horizontal ) {
			subFigure1.setPreferredSize( new Dimension( (w * splitPercentage) / 100, h ) );
			subFigure2.setPreferredSize( new Dimension( (w * (100-splitPercentage)) / 100, h ) );
		} else {
			subFigure1.setPreferredSize( new Dimension( w, (h * splitPercentage) / 100 ) );
			subFigure2.setPreferredSize( new Dimension( w, (h * (100-splitPercentage)) / 100 ) );
		}
	}
	
	/**
	 * Says that this figure uses local coordinates.
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}
	

	/**
	 * Returns the minimum size of this ConstraintFigure
	 * based on the size of the context, expression and condition figures.
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) 
	{
		Dimension d;
		
		Dimension dim1 = subFigure1.getMinimumSize();
		Dimension dim2 = subFigure2.getMinimumSize();
		if ( horizontal )
			d = new Dimension( dim1.width + dim2.width, Math.max( dim1.height, dim2.height ) );
		else
			d = new Dimension( Math.max( dim1.width, dim2.width), dim1.height + dim2.height );

		return d;
	}
}