/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.model;

import java.util.Map;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.gef.requests.CreationFactory;

import VOCLModel.*;


/**
 * This class implements the CreationFactory used by the CreationTool. It in turn
 * uses the EMF-generated factories to create the model instances
 */
public class ModelCreationFactory implements CreationFactory {
	private Class targetClass;
	
	public ModelCreationFactory( Class targetClass ) {
		this.targetClass = targetClass;
	}
	/**
	 * Returns the Objects associated with the c.
	 * Object creation are made through that factory.
	 * @return the Object
	 */
	public Object getNewObject() {
		Map registry = EPackage.Registry.INSTANCE;
		String voclmodelURI = VOCLModelPackage.eNS_URI;
		VOCLModelPackage voclmodelPackage =
		(VOCLModelPackage) registry.get(voclmodelURI);
		VOCLModelFactory factory = voclmodelPackage.getVOCLModelFactory();
			
		Object result = null;
		if (targetClass.equals(AssociationRole.class)){
			result = factory.createAssociationRole();
		}
		else if (targetClass.equals(Attribute.class)){
			result = factory.createAttribute();
		}
		else if (targetClass.equals(SetClassifierRole.class)){
			result = factory.createSetClassifierRole();
		}
		else if (targetClass.equals(ClassifierRole.class)){
			result = factory.createClassifierRole();
		}
		else if (targetClass.equals(Method.class)) {
			result = factory.createMethod();
		}		
		else if (targetClass.equals(VariableDeclaration.class)) {
			result = factory.createVariableDeclaration();
		}
		else if (targetClass.equals(Condition.class)) {
			result = factory.createCondition();
		}
		else if (targetClass.equals(Context.class)) {
			result = factory.createContext();
		}
		else if (targetClass.equals(BoolExp.class)) {
			BoolExp boolexp = (BoolExp) factory.createBoolExp();
			boolexp.setOp1( factory.createNavExp() );
			boolexp.setOp2( factory.createNavExp() );
			boolexp.setSplitPercent( 50 );
			result = boolexp;
		}
		else if (targetClass.equals(LetExp.class)) {
			LetExp letexp = (LetExp) factory.createLetExp();
			letexp.setLet( factory.createNavExp() );
			letexp.setIn( factory.createNavExp() );
			letexp.setSplitPercent( 50 );
			result = letexp;
		}
		else if (targetClass.equals(IfExp.class)) {
			IfExp ifexp = (IfExp) factory.createIfExp();
			ifexp.setIfExp( factory.createNavExp() );
			ifexp.setThenExp( factory.createNavExp() );
			ifexp.setElseExp( factory.createNavExp() );
			ifexp.setSplitPercent1( 50 );
			ifexp.setSplitPercent2( 50 );
			result = ifexp;
		}
		else if (targetClass.equals(Constraint.class)) {
			Constraint constraint = factory.createConstraint();
			constraint.setWidth( 320 );
			constraint.setHeight( 200 );
			Context context = factory.createContext();
			context.setType( "undefined" );
			constraint.setContext( context );
			constraint.setExpression( factory.createNavExp() );
			result = constraint;
		} else if(targetClass.equals(Label.class)) {
			result = factory.createLabel();
		} else
			throw new IllegalArgumentException("ModelCreationFactory: unknown class: " + targetClass.getName());
		return result;
	}
	/**
	 * inherited abstract method
	 * @return object
	 */
	public Object getObjectType() {
		return targetClass;
	}
}
